// Copyright 2016, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// -----------------------------------------------------------------------------
// This file is auto generated from the
// test/aarch32/config/template-assembler-aarch32.cc.in template file using
// tools/generate_tests.py.
//
// PLEASE DO NOT EDIT.
// -----------------------------------------------------------------------------


#include "test-runner.h"

#include "test-utils.h"
#include "test-utils-aarch32.h"

#include "aarch32/assembler-aarch32.h"
#include "aarch32/macro-assembler-aarch32.h"

#define BUF_SIZE (4096)

namespace vixl {
namespace aarch32 {

// List of instruction mnemonics.
#define FOREACH_INSTRUCTION(M) M(mov)


// The following definitions are defined again in each generated test, therefore
// we need to place them in an anomymous namespace. It expresses that they are
// local to this file only, and the compiler is not allowed to share these types
// across test files during template instantiation. Specifically, `Operands` has
// various layouts across generated tests so it absolutely cannot be shared.

#ifdef VIXL_INCLUDE_TARGET_T32
namespace {

// Values to be passed to the assembler to produce the instruction under test.
struct Operands {
  Condition cond;
  Register rd;
  Register rn;
  ShiftType shift;
  Register rs;
};

// This structure contains all data needed to test one specific
// instruction.
struct TestData {
  // The `operands` field represents what to pass to the assembler to
  // produce the instruction.
  Operands operands;
  // True if we need to generate an IT instruction for this test to be valid.
  bool in_it_block;
  // The condition to give the IT instruction, this will be set to "al" by
  // default.
  Condition it_condition;
  // Description of the operands, used for error reporting.
  const char* operands_description;
  // Unique identifier, used for generating traces.
  const char* identifier;
};

struct TestResult {
  size_t size;
  const byte* encoding;
};

// Each element of this array produce one instruction encoding.
const TestData kTests[] =
    {{{ge, r1, r1, LSL, r6}, true, ge, "ge r1 r1 LSL r6", "ge_r1_r1_LSL_r6"},
     {{cs, r7, r7, LSL, r2}, true, cs, "cs r7 r7 LSL r2", "cs_r7_r7_LSL_r2"},
     {{gt, r1, r1, LSL, r0}, true, gt, "gt r1 r1 LSL r0", "gt_r1_r1_LSL_r0"},
     {{pl, r3, r3, ROR, r3}, true, pl, "pl r3 r3 ROR r3", "pl_r3_r3_ROR_r3"},
     {{vc, r4, r4, ROR, r1}, true, vc, "vc r4 r4 ROR r1", "vc_r4_r4_ROR_r1"},
     {{vs, r2, r2, LSL, r0}, true, vs, "vs r2 r2 LSL r0", "vs_r2_r2_LSL_r0"},
     {{vc, r4, r4, LSR, r4}, true, vc, "vc r4 r4 LSR r4", "vc_r4_r4_LSR_r4"},
     {{le, r6, r6, ASR, r6}, true, le, "le r6 r6 ASR r6", "le_r6_r6_ASR_r6"},
     {{hi, r6, r6, ROR, r5}, true, hi, "hi r6 r6 ROR r5", "hi_r6_r6_ROR_r5"},
     {{le, r3, r3, ASR, r6}, true, le, "le r3 r3 ASR r6", "le_r3_r3_ASR_r6"},
     {{le, r7, r7, LSL, r0}, true, le, "le r7 r7 LSL r0", "le_r7_r7_LSL_r0"},
     {{pl, r3, r3, ASR, r2}, true, pl, "pl r3 r3 ASR r2", "pl_r3_r3_ASR_r2"},
     {{mi, r4, r4, LSL, r5}, true, mi, "mi r4 r4 LSL r5", "mi_r4_r4_LSL_r5"},
     {{le, r5, r5, LSR, r6}, true, le, "le r5 r5 LSR r6", "le_r5_r5_LSR_r6"},
     {{cs, r0, r0, ASR, r6}, true, cs, "cs r0 r0 ASR r6", "cs_r0_r0_ASR_r6"},
     {{vs, r2, r2, LSL, r1}, true, vs, "vs r2 r2 LSL r1", "vs_r2_r2_LSL_r1"},
     {{hi, r5, r5, LSL, r6}, true, hi, "hi r5 r5 LSL r6", "hi_r5_r5_LSL_r6"},
     {{ls, r6, r6, LSL, r3}, true, ls, "ls r6 r6 LSL r3", "ls_r6_r6_LSL_r3"},
     {{pl, r2, r2, ASR, r7}, true, pl, "pl r2 r2 ASR r7", "pl_r2_r2_ASR_r7"},
     {{ls, r0, r0, LSR, r2}, true, ls, "ls r0 r0 LSR r2", "ls_r0_r0_LSR_r2"},
     {{eq, r5, r5, ROR, r2}, true, eq, "eq r5 r5 ROR r2", "eq_r5_r5_ROR_r2"},
     {{eq, r5, r5, LSR, r1}, true, eq, "eq r5 r5 LSR r1", "eq_r5_r5_LSR_r1"},
     {{ge, r4, r4, ASR, r1}, true, ge, "ge r4 r4 ASR r1", "ge_r4_r4_ASR_r1"},
     {{ls, r6, r6, LSR, r0}, true, ls, "ls r6 r6 LSR r0", "ls_r6_r6_LSR_r0"},
     {{ls, r0, r0, ASR, r2}, true, ls, "ls r0 r0 ASR r2", "ls_r0_r0_ASR_r2"},
     {{cc, r1, r1, LSL, r6}, true, cc, "cc r1 r1 LSL r6", "cc_r1_r1_LSL_r6"},
     {{ge, r6, r6, ROR, r3}, true, ge, "ge r6 r6 ROR r3", "ge_r6_r6_ROR_r3"},
     {{cs, r7, r7, ASR, r3}, true, cs, "cs r7 r7 ASR r3", "cs_r7_r7_ASR_r3"},
     {{ne, r3, r3, ROR, r4}, true, ne, "ne r3 r3 ROR r4", "ne_r3_r3_ROR_r4"},
     {{hi, r1, r1, ASR, r6}, true, hi, "hi r1 r1 ASR r6", "hi_r1_r1_ASR_r6"},
     {{ls, r7, r7, LSL, r4}, true, ls, "ls r7 r7 LSL r4", "ls_r7_r7_LSL_r4"},
     {{ge, r4, r4, LSL, r0}, true, ge, "ge r4 r4 LSL r0", "ge_r4_r4_LSL_r0"},
     {{vs, r4, r4, LSR, r4}, true, vs, "vs r4 r4 LSR r4", "vs_r4_r4_LSR_r4"},
     {{cc, r0, r0, LSR, r0}, true, cc, "cc r0 r0 LSR r0", "cc_r0_r0_LSR_r0"},
     {{mi, r5, r5, LSL, r5}, true, mi, "mi r5 r5 LSL r5", "mi_r5_r5_LSL_r5"},
     {{ls, r1, r1, LSR, r0}, true, ls, "ls r1 r1 LSR r0", "ls_r1_r1_LSR_r0"},
     {{ge, r1, r1, ASR, r7}, true, ge, "ge r1 r1 ASR r7", "ge_r1_r1_ASR_r7"},
     {{eq, r4, r4, ROR, r4}, true, eq, "eq r4 r4 ROR r4", "eq_r4_r4_ROR_r4"},
     {{vc, r5, r5, ROR, r3}, true, vc, "vc r5 r5 ROR r3", "vc_r5_r5_ROR_r3"},
     {{ls, r3, r3, LSL, r2}, true, ls, "ls r3 r3 LSL r2", "ls_r3_r3_LSL_r2"},
     {{ls, r6, r6, ASR, r3}, true, ls, "ls r6 r6 ASR r3", "ls_r6_r6_ASR_r3"},
     {{lt, r4, r4, ROR, r3}, true, lt, "lt r4 r4 ROR r3", "lt_r4_r4_ROR_r3"},
     {{vc, r6, r6, ASR, r7}, true, vc, "vc r6 r6 ASR r7", "vc_r6_r6_ASR_r7"},
     {{ls, r1, r1, LSR, r3}, true, ls, "ls r1 r1 LSR r3", "ls_r1_r1_LSR_r3"},
     {{ne, r6, r6, ROR, r6}, true, ne, "ne r6 r6 ROR r6", "ne_r6_r6_ROR_r6"},
     {{cc, r3, r3, LSL, r7}, true, cc, "cc r3 r3 LSL r7", "cc_r3_r3_LSL_r7"},
     {{hi, r3, r3, LSR, r0}, true, hi, "hi r3 r3 LSR r0", "hi_r3_r3_LSR_r0"},
     {{gt, r5, r5, ROR, r0}, true, gt, "gt r5 r5 ROR r0", "gt_r5_r5_ROR_r0"},
     {{cc, r3, r3, LSL, r0}, true, cc, "cc r3 r3 LSL r0", "cc_r3_r3_LSL_r0"},
     {{le, r1, r1, LSL, r0}, true, le, "le r1 r1 LSL r0", "le_r1_r1_LSL_r0"},
     {{vs, r5, r5, ASR, r2}, true, vs, "vs r5 r5 ASR r2", "vs_r5_r5_ASR_r2"},
     {{mi, r2, r2, ASR, r6}, true, mi, "mi r2 r2 ASR r6", "mi_r2_r2_ASR_r6"},
     {{ls, r7, r7, LSR, r5}, true, ls, "ls r7 r7 LSR r5", "ls_r7_r7_LSR_r5"},
     {{lt, r0, r0, LSR, r0}, true, lt, "lt r0 r0 LSR r0", "lt_r0_r0_LSR_r0"},
     {{mi, r6, r6, ROR, r1}, true, mi, "mi r6 r6 ROR r1", "mi_r6_r6_ROR_r1"},
     {{cs, r1, r1, ASR, r3}, true, cs, "cs r1 r1 ASR r3", "cs_r1_r1_ASR_r3"},
     {{le, r0, r0, ROR, r3}, true, le, "le r0 r0 ROR r3", "le_r0_r0_ROR_r3"},
     {{mi, r7, r7, LSR, r5}, true, mi, "mi r7 r7 LSR r5", "mi_r7_r7_LSR_r5"},
     {{mi, r0, r0, LSL, r7}, true, mi, "mi r0 r0 LSL r7", "mi_r0_r0_LSL_r7"},
     {{vc, r3, r3, LSR, r7}, true, vc, "vc r3 r3 LSR r7", "vc_r3_r3_LSR_r7"},
     {{ge, r3, r3, ROR, r7}, true, ge, "ge r3 r3 ROR r7", "ge_r3_r3_ROR_r7"},
     {{vc, r1, r1, LSL, r7}, true, vc, "vc r1 r1 LSL r7", "vc_r1_r1_LSL_r7"},
     {{hi, r3, r3, LSL, r3}, true, hi, "hi r3 r3 LSL r3", "hi_r3_r3_LSL_r3"},
     {{eq, r1, r1, ASR, r5}, true, eq, "eq r1 r1 ASR r5", "eq_r1_r1_ASR_r5"},
     {{lt, r5, r5, LSR, r4}, true, lt, "lt r5 r5 LSR r4", "lt_r5_r5_LSR_r4"},
     {{gt, r1, r1, ASR, r4}, true, gt, "gt r1 r1 ASR r4", "gt_r1_r1_ASR_r4"},
     {{ge, r4, r4, ROR, r1}, true, ge, "ge r4 r4 ROR r1", "ge_r4_r4_ROR_r1"},
     {{vs, r5, r5, LSL, r0}, true, vs, "vs r5 r5 LSL r0", "vs_r5_r5_LSL_r0"},
     {{eq, r0, r0, ASR, r4}, true, eq, "eq r0 r0 ASR r4", "eq_r0_r0_ASR_r4"},
     {{ge, r3, r3, LSL, r4}, true, ge, "ge r3 r3 LSL r4", "ge_r3_r3_LSL_r4"},
     {{pl, r3, r3, ASR, r6}, true, pl, "pl r3 r3 ASR r6", "pl_r3_r3_ASR_r6"},
     {{cc, r2, r2, LSL, r3}, true, cc, "cc r2 r2 LSL r3", "cc_r2_r2_LSL_r3"},
     {{pl, r3, r3, ASR, r1}, true, pl, "pl r3 r3 ASR r1", "pl_r3_r3_ASR_r1"},
     {{vc, r2, r2, LSR, r7}, true, vc, "vc r2 r2 LSR r7", "vc_r2_r2_LSR_r7"},
     {{ge, r1, r1, ASR, r2}, true, ge, "ge r1 r1 ASR r2", "ge_r1_r1_ASR_r2"},
     {{ge, r7, r7, LSR, r4}, true, ge, "ge r7 r7 LSR r4", "ge_r7_r7_LSR_r4"},
     {{ls, r5, r5, LSR, r3}, true, ls, "ls r5 r5 LSR r3", "ls_r5_r5_LSR_r3"},
     {{eq, r4, r4, LSR, r5}, true, eq, "eq r4 r4 LSR r5", "eq_r4_r4_LSR_r5"},
     {{ne, r7, r7, ROR, r6}, true, ne, "ne r7 r7 ROR r6", "ne_r7_r7_ROR_r6"},
     {{mi, r5, r5, LSL, r7}, true, mi, "mi r5 r5 LSL r7", "mi_r5_r5_LSL_r7"},
     {{cs, r7, r7, ROR, r4}, true, cs, "cs r7 r7 ROR r4", "cs_r7_r7_ROR_r4"},
     {{mi, r1, r1, ASR, r6}, true, mi, "mi r1 r1 ASR r6", "mi_r1_r1_ASR_r6"},
     {{pl, r6, r6, ASR, r0}, true, pl, "pl r6 r6 ASR r0", "pl_r6_r6_ASR_r0"},
     {{cs, r3, r3, ASR, r2}, true, cs, "cs r3 r3 ASR r2", "cs_r3_r3_ASR_r2"},
     {{eq, r2, r2, LSL, r1}, true, eq, "eq r2 r2 LSL r1", "eq_r2_r2_LSL_r1"},
     {{lt, r6, r6, LSR, r5}, true, lt, "lt r6 r6 LSR r5", "lt_r6_r6_LSR_r5"},
     {{cc, r0, r0, ASR, r5}, true, cc, "cc r0 r0 ASR r5", "cc_r0_r0_ASR_r5"},
     {{cs, r1, r1, LSL, r7}, true, cs, "cs r1 r1 LSL r7", "cs_r1_r1_LSL_r7"},
     {{ge, r0, r0, LSR, r0}, true, ge, "ge r0 r0 LSR r0", "ge_r0_r0_LSR_r0"},
     {{cc, r4, r4, ROR, r7}, true, cc, "cc r4 r4 ROR r7", "cc_r4_r4_ROR_r7"},
     {{cs, r5, r5, ASR, r4}, true, cs, "cs r5 r5 ASR r4", "cs_r5_r5_ASR_r4"},
     {{vc, r6, r6, LSL, r0}, true, vc, "vc r6 r6 LSL r0", "vc_r6_r6_LSL_r0"},
     {{gt, r5, r5, LSL, r4}, true, gt, "gt r5 r5 LSL r4", "gt_r5_r5_LSL_r4"},
     {{hi, r2, r2, ROR, r3}, true, hi, "hi r2 r2 ROR r3", "hi_r2_r2_ROR_r3"},
     {{cc, r4, r4, ROR, r3}, true, cc, "cc r4 r4 ROR r3", "cc_r4_r4_ROR_r3"},
     {{hi, r0, r0, ROR, r4}, true, hi, "hi r0 r0 ROR r4", "hi_r0_r0_ROR_r4"},
     {{vs, r1, r1, ROR, r3}, true, vs, "vs r1 r1 ROR r3", "vs_r1_r1_ROR_r3"},
     {{gt, r4, r4, ROR, r5}, true, gt, "gt r4 r4 ROR r5", "gt_r4_r4_ROR_r5"},
     {{lt, r1, r1, LSR, r2}, true, lt, "lt r1 r1 LSR r2", "lt_r1_r1_LSR_r2"},
     {{lt, r4, r4, LSL, r4}, true, lt, "lt r4 r4 LSL r4", "lt_r4_r4_LSL_r4"},
     {{mi, r3, r3, LSL, r2}, true, mi, "mi r3 r3 LSL r2", "mi_r3_r3_LSL_r2"},
     {{cs, r6, r6, ROR, r7}, true, cs, "cs r6 r6 ROR r7", "cs_r6_r6_ROR_r7"},
     {{vc, r5, r5, LSR, r3}, true, vc, "vc r5 r5 LSR r3", "vc_r5_r5_LSR_r3"},
     {{vc, r4, r4, LSL, r4}, true, vc, "vc r4 r4 LSL r4", "vc_r4_r4_LSL_r4"},
     {{pl, r0, r0, LSL, r2}, true, pl, "pl r0 r0 LSL r2", "pl_r0_r0_LSL_r2"},
     {{ne, r3, r3, LSL, r5}, true, ne, "ne r3 r3 LSL r5", "ne_r3_r3_LSL_r5"},
     {{mi, r7, r7, ROR, r5}, true, mi, "mi r7 r7 ROR r5", "mi_r7_r7_ROR_r5"},
     {{ls, r4, r4, LSL, r4}, true, ls, "ls r4 r4 LSL r4", "ls_r4_r4_LSL_r4"},
     {{hi, r0, r0, ASR, r1}, true, hi, "hi r0 r0 ASR r1", "hi_r0_r0_ASR_r1"},
     {{vs, r5, r5, ROR, r0}, true, vs, "vs r5 r5 ROR r0", "vs_r5_r5_ROR_r0"},
     {{hi, r1, r1, ROR, r4}, true, hi, "hi r1 r1 ROR r4", "hi_r1_r1_ROR_r4"},
     {{vs, r7, r7, ROR, r2}, true, vs, "vs r7 r7 ROR r2", "vs_r7_r7_ROR_r2"},
     {{gt, r5, r5, ROR, r2}, true, gt, "gt r5 r5 ROR r2", "gt_r5_r5_ROR_r2"},
     {{ge, r0, r0, LSR, r7}, true, ge, "ge r0 r0 LSR r7", "ge_r0_r0_LSR_r7"},
     {{cs, r7, r7, LSR, r0}, true, cs, "cs r7 r7 LSR r0", "cs_r7_r7_LSR_r0"},
     {{ge, r1, r1, LSL, r3}, true, ge, "ge r1 r1 LSL r3", "ge_r1_r1_LSL_r3"},
     {{lt, r5, r5, LSL, r5}, true, lt, "lt r5 r5 LSL r5", "lt_r5_r5_LSL_r5"},
     {{ls, r3, r3, ROR, r0}, true, ls, "ls r3 r3 ROR r0", "ls_r3_r3_ROR_r0"},
     {{cs, r0, r0, LSL, r7}, true, cs, "cs r0 r0 LSL r7", "cs_r0_r0_LSL_r7"},
     {{pl, r7, r7, ASR, r0}, true, pl, "pl r7 r7 ASR r0", "pl_r7_r7_ASR_r0"},
     {{vs, r2, r2, LSL, r3}, true, vs, "vs r2 r2 LSL r3", "vs_r2_r2_LSL_r3"},
     {{ne, r0, r0, ROR, r1}, true, ne, "ne r0 r0 ROR r1", "ne_r0_r0_ROR_r1"},
     {{vs, r2, r2, ASR, r2}, true, vs, "vs r2 r2 ASR r2", "vs_r2_r2_ASR_r2"},
     {{pl, r6, r6, LSR, r3}, true, pl, "pl r6 r6 LSR r3", "pl_r6_r6_LSR_r3"},
     {{gt, r3, r3, ROR, r0}, true, gt, "gt r3 r3 ROR r0", "gt_r3_r3_ROR_r0"},
     {{vc, r3, r3, LSL, r7}, true, vc, "vc r3 r3 LSL r7", "vc_r3_r3_LSL_r7"},
     {{ge, r6, r6, LSL, r6}, true, ge, "ge r6 r6 LSL r6", "ge_r6_r6_LSL_r6"},
     {{ls, r2, r2, LSL, r3}, true, ls, "ls r2 r2 LSL r3", "ls_r2_r2_LSL_r3"},
     {{vc, r4, r4, ROR, r5}, true, vc, "vc r4 r4 ROR r5", "vc_r4_r4_ROR_r5"},
     {{hi, r7, r7, LSL, r1}, true, hi, "hi r7 r7 LSL r1", "hi_r7_r7_LSL_r1"},
     {{cs, r1, r1, ROR, r2}, true, cs, "cs r1 r1 ROR r2", "cs_r1_r1_ROR_r2"},
     {{vs, r1, r1, ROR, r7}, true, vs, "vs r1 r1 ROR r7", "vs_r1_r1_ROR_r7"},
     {{lt, r3, r3, ASR, r2}, true, lt, "lt r3 r3 ASR r2", "lt_r3_r3_ASR_r2"},
     {{ne, r0, r0, LSR, r7}, true, ne, "ne r0 r0 LSR r7", "ne_r0_r0_LSR_r7"},
     {{vs, r6, r6, ROR, r1}, true, vs, "vs r6 r6 ROR r1", "vs_r6_r6_ROR_r1"},
     {{hi, r3, r3, ROR, r1}, true, hi, "hi r3 r3 ROR r1", "hi_r3_r3_ROR_r1"},
     {{ls, r1, r1, ASR, r0}, true, ls, "ls r1 r1 ASR r0", "ls_r1_r1_ASR_r0"},
     {{cs, r1, r1, LSL, r0}, true, cs, "cs r1 r1 LSL r0", "cs_r1_r1_LSL_r0"},
     {{cc, r7, r7, ROR, r6}, true, cc, "cc r7 r7 ROR r6", "cc_r7_r7_ROR_r6"},
     {{mi, r2, r2, LSL, r5}, true, mi, "mi r2 r2 LSL r5", "mi_r2_r2_LSL_r5"},
     {{gt, r6, r6, LSL, r5}, true, gt, "gt r6 r6 LSL r5", "gt_r6_r6_LSL_r5"},
     {{gt, r4, r4, LSL, r5}, true, gt, "gt r4 r4 LSL r5", "gt_r4_r4_LSL_r5"},
     {{vc, r3, r3, LSL, r4}, true, vc, "vc r3 r3 LSL r4", "vc_r3_r3_LSL_r4"},
     {{gt, r7, r7, LSR, r2}, true, gt, "gt r7 r7 LSR r2", "gt_r7_r7_LSR_r2"},
     {{le, r5, r5, ROR, r2}, true, le, "le r5 r5 ROR r2", "le_r5_r5_ROR_r2"},
     {{vs, r6, r6, LSL, r3}, true, vs, "vs r6 r6 LSL r3", "vs_r6_r6_LSL_r3"},
     {{gt, r1, r1, LSL, r6}, true, gt, "gt r1 r1 LSL r6", "gt_r1_r1_LSL_r6"},
     {{cc, r0, r0, LSR, r5}, true, cc, "cc r0 r0 LSR r5", "cc_r0_r0_LSR_r5"},
     {{eq, r4, r4, LSR, r3}, true, eq, "eq r4 r4 LSR r3", "eq_r4_r4_LSR_r3"},
     {{hi, r6, r6, LSR, r6}, true, hi, "hi r6 r6 LSR r6", "hi_r6_r6_LSR_r6"},
     {{hi, r5, r5, LSR, r4}, true, hi, "hi r5 r5 LSR r4", "hi_r5_r5_LSR_r4"},
     {{vs, r6, r6, ASR, r0}, true, vs, "vs r6 r6 ASR r0", "vs_r6_r6_ASR_r0"},
     {{hi, r6, r6, LSL, r6}, true, hi, "hi r6 r6 LSL r6", "hi_r6_r6_LSL_r6"},
     {{eq, r6, r6, ROR, r1}, true, eq, "eq r6 r6 ROR r1", "eq_r6_r6_ROR_r1"},
     {{ne, r1, r1, LSL, r5}, true, ne, "ne r1 r1 LSL r5", "ne_r1_r1_LSL_r5"},
     {{lt, r0, r0, ASR, r4}, true, lt, "lt r0 r0 ASR r4", "lt_r0_r0_ASR_r4"},
     {{le, r2, r2, LSL, r0}, true, le, "le r2 r2 LSL r0", "le_r2_r2_LSL_r0"},
     {{ls, r4, r4, ASR, r2}, true, ls, "ls r4 r4 ASR r2", "ls_r4_r4_ASR_r2"},
     {{pl, r4, r4, ASR, r2}, true, pl, "pl r4 r4 ASR r2", "pl_r4_r4_ASR_r2"},
     {{lt, r6, r6, ROR, r3}, true, lt, "lt r6 r6 ROR r3", "lt_r6_r6_ROR_r3"},
     {{ne, r4, r4, ROR, r5}, true, ne, "ne r4 r4 ROR r5", "ne_r4_r4_ROR_r5"},
     {{cc, r7, r7, LSL, r7}, true, cc, "cc r7 r7 LSL r7", "cc_r7_r7_LSL_r7"},
     {{le, r7, r7, LSR, r6}, true, le, "le r7 r7 LSR r6", "le_r7_r7_LSR_r6"},
     {{vc, r4, r4, LSL, r7}, true, vc, "vc r4 r4 LSL r7", "vc_r4_r4_LSL_r7"},
     {{vs, r1, r1, LSL, r3}, true, vs, "vs r1 r1 LSL r3", "vs_r1_r1_LSL_r3"},
     {{eq, r3, r3, LSL, r2}, true, eq, "eq r3 r3 LSL r2", "eq_r3_r3_LSL_r2"},
     {{ls, r7, r7, LSL, r3}, true, ls, "ls r7 r7 LSL r3", "ls_r7_r7_LSL_r3"},
     {{lt, r2, r2, ROR, r3}, true, lt, "lt r2 r2 ROR r3", "lt_r2_r2_ROR_r3"},
     {{lt, r2, r2, LSR, r2}, true, lt, "lt r2 r2 LSR r2", "lt_r2_r2_LSR_r2"},
     {{eq, r0, r0, LSL, r5}, true, eq, "eq r0 r0 LSL r5", "eq_r0_r0_LSL_r5"},
     {{vc, r2, r2, LSR, r1}, true, vc, "vc r2 r2 LSR r1", "vc_r2_r2_LSR_r1"},
     {{ge, r6, r6, ROR, r2}, true, ge, "ge r6 r6 ROR r2", "ge_r6_r6_ROR_r2"},
     {{ls, r1, r1, LSL, r4}, true, ls, "ls r1 r1 LSL r4", "ls_r1_r1_LSL_r4"},
     {{le, r6, r6, ROR, r2}, true, le, "le r6 r6 ROR r2", "le_r6_r6_ROR_r2"},
     {{vc, r6, r6, LSL, r5}, true, vc, "vc r6 r6 LSL r5", "vc_r6_r6_LSL_r5"},
     {{le, r0, r0, LSL, r5}, true, le, "le r0 r0 LSL r5", "le_r0_r0_LSL_r5"},
     {{vc, r3, r3, LSR, r1}, true, vc, "vc r3 r3 LSR r1", "vc_r3_r3_LSR_r1"},
     {{ge, r2, r2, LSR, r0}, true, ge, "ge r2 r2 LSR r0", "ge_r2_r2_LSR_r0"},
     {{vc, r4, r4, LSR, r0}, true, vc, "vc r4 r4 LSR r0", "vc_r4_r4_LSR_r0"},
     {{pl, r7, r7, LSR, r0}, true, pl, "pl r7 r7 LSR r0", "pl_r7_r7_LSR_r0"},
     {{vs, r7, r7, ROR, r4}, true, vs, "vs r7 r7 ROR r4", "vs_r7_r7_ROR_r4"},
     {{cs, r6, r6, ASR, r5}, true, cs, "cs r6 r6 ASR r5", "cs_r6_r6_ASR_r5"},
     {{ls, r6, r6, ROR, r4}, true, ls, "ls r6 r6 ROR r4", "ls_r6_r6_ROR_r4"},
     {{pl, r2, r2, LSR, r4}, true, pl, "pl r2 r2 LSR r4", "pl_r2_r2_LSR_r4"},
     {{lt, r4, r4, ASR, r0}, true, lt, "lt r4 r4 ASR r0", "lt_r4_r4_ASR_r0"},
     {{vc, r0, r0, LSL, r6}, true, vc, "vc r0 r0 LSL r6", "vc_r0_r0_LSL_r6"},
     {{lt, r7, r7, LSL, r6}, true, lt, "lt r7 r7 LSL r6", "lt_r7_r7_LSL_r6"},
     {{gt, r2, r2, LSL, r2}, true, gt, "gt r2 r2 LSL r2", "gt_r2_r2_LSL_r2"},
     {{vc, r2, r2, ROR, r7}, true, vc, "vc r2 r2 ROR r7", "vc_r2_r2_ROR_r7"},
     {{cc, r5, r5, ASR, r6}, true, cc, "cc r5 r5 ASR r6", "cc_r5_r5_ASR_r6"},
     {{cc, r0, r0, LSL, r0}, true, cc, "cc r0 r0 LSL r0", "cc_r0_r0_LSL_r0"},
     {{vc, r4, r4, ROR, r6}, true, vc, "vc r4 r4 ROR r6", "vc_r4_r4_ROR_r6"},
     {{gt, r6, r6, LSR, r3}, true, gt, "gt r6 r6 LSR r3", "gt_r6_r6_LSR_r3"},
     {{ne, r5, r5, LSR, r1}, true, ne, "ne r5 r5 LSR r1", "ne_r5_r5_LSR_r1"},
     {{ne, r3, r3, ROR, r3}, true, ne, "ne r3 r3 ROR r3", "ne_r3_r3_ROR_r3"},
     {{ge, r3, r3, LSL, r2}, true, ge, "ge r3 r3 LSL r2", "ge_r3_r3_LSL_r2"},
     {{pl, r3, r3, LSL, r2}, true, pl, "pl r3 r3 LSL r2", "pl_r3_r3_LSL_r2"},
     {{eq, r7, r7, LSL, r2}, true, eq, "eq r7 r7 LSL r2", "eq_r7_r7_LSL_r2"},
     {{pl, r6, r6, LSL, r7}, true, pl, "pl r6 r6 LSL r7", "pl_r6_r6_LSL_r7"},
     {{ls, r2, r2, LSL, r0}, true, ls, "ls r2 r2 LSL r0", "ls_r2_r2_LSL_r0"},
     {{cs, r7, r7, LSL, r1}, true, cs, "cs r7 r7 LSL r1", "cs_r7_r7_LSL_r1"},
     {{mi, r7, r7, LSR, r4}, true, mi, "mi r7 r7 LSR r4", "mi_r7_r7_LSR_r4"},
     {{gt, r7, r7, ROR, r4}, true, gt, "gt r7 r7 ROR r4", "gt_r7_r7_ROR_r4"},
     {{pl, r7, r7, LSL, r3}, true, pl, "pl r7 r7 LSL r3", "pl_r7_r7_LSL_r3"},
     {{cc, r7, r7, ASR, r3}, true, cc, "cc r7 r7 ASR r3", "cc_r7_r7_ASR_r3"},
     {{hi, r6, r6, ASR, r0}, true, hi, "hi r6 r6 ASR r0", "hi_r6_r6_ASR_r0"},
     {{ge, r0, r0, ROR, r5}, true, ge, "ge r0 r0 ROR r5", "ge_r0_r0_ROR_r5"},
     {{ne, r3, r3, LSR, r1}, true, ne, "ne r3 r3 LSR r1", "ne_r3_r3_LSR_r1"},
     {{le, r0, r0, ROR, r1}, true, le, "le r0 r0 ROR r1", "le_r0_r0_ROR_r1"},
     {{cc, r6, r6, LSL, r0}, true, cc, "cc r6 r6 LSL r0", "cc_r6_r6_LSL_r0"},
     {{eq, r4, r4, LSL, r5}, true, eq, "eq r4 r4 LSL r5", "eq_r4_r4_LSL_r5"},
     {{le, r3, r3, ASR, r7}, true, le, "le r3 r3 ASR r7", "le_r3_r3_ASR_r7"},
     {{pl, r6, r6, ROR, r6}, true, pl, "pl r6 r6 ROR r6", "pl_r6_r6_ROR_r6"},
     {{gt, r6, r6, LSR, r4}, true, gt, "gt r6 r6 LSR r4", "gt_r6_r6_LSR_r4"},
     {{hi, r2, r2, LSL, r0}, true, hi, "hi r2 r2 LSL r0", "hi_r2_r2_LSL_r0"},
     {{eq, r1, r1, LSR, r7}, true, eq, "eq r1 r1 LSR r7", "eq_r1_r1_LSR_r7"},
     {{eq, r1, r1, LSR, r1}, true, eq, "eq r1 r1 LSR r1", "eq_r1_r1_LSR_r1"},
     {{vs, r5, r5, ASR, r5}, true, vs, "vs r5 r5 ASR r5", "vs_r5_r5_ASR_r5"},
     {{gt, r0, r0, LSR, r0}, true, gt, "gt r0 r0 LSR r0", "gt_r0_r0_LSR_r0"},
     {{hi, r7, r7, ROR, r2}, true, hi, "hi r7 r7 ROR r2", "hi_r7_r7_ROR_r2"},
     {{vc, r0, r0, ROR, r7}, true, vc, "vc r0 r0 ROR r7", "vc_r0_r0_ROR_r7"},
     {{eq, r2, r2, ASR, r2}, true, eq, "eq r2 r2 ASR r2", "eq_r2_r2_ASR_r2"},
     {{vc, r0, r0, LSR, r3}, true, vc, "vc r0 r0 LSR r3", "vc_r0_r0_LSR_r3"},
     {{le, r7, r7, ASR, r2}, true, le, "le r7 r7 ASR r2", "le_r7_r7_ASR_r2"},
     {{ls, r4, r4, ASR, r6}, true, ls, "ls r4 r4 ASR r6", "ls_r4_r4_ASR_r6"},
     {{pl, r1, r1, ROR, r5}, true, pl, "pl r1 r1 ROR r5", "pl_r1_r1_ROR_r5"},
     {{le, r4, r4, ASR, r2}, true, le, "le r4 r4 ASR r2", "le_r4_r4_ASR_r2"},
     {{mi, r7, r7, LSL, r6}, true, mi, "mi r7 r7 LSL r6", "mi_r7_r7_LSL_r6"},
     {{cc, r0, r0, ROR, r6}, true, cc, "cc r0 r0 ROR r6", "cc_r0_r0_ROR_r6"},
     {{le, r6, r6, LSL, r7}, true, le, "le r6 r6 LSL r7", "le_r6_r6_LSL_r7"},
     {{hi, r0, r0, ASR, r0}, true, hi, "hi r0 r0 ASR r0", "hi_r0_r0_ASR_r0"},
     {{cs, r7, r7, ASR, r5}, true, cs, "cs r7 r7 ASR r5", "cs_r7_r7_ASR_r5"},
     {{eq, r2, r2, ASR, r7}, true, eq, "eq r2 r2 ASR r7", "eq_r2_r2_ASR_r7"},
     {{eq, r5, r5, LSL, r0}, true, eq, "eq r5 r5 LSL r0", "eq_r5_r5_LSL_r0"},
     {{hi, r5, r5, ASR, r6}, true, hi, "hi r5 r5 ASR r6", "hi_r5_r5_ASR_r6"},
     {{ne, r3, r3, ASR, r7}, true, ne, "ne r3 r3 ASR r7", "ne_r3_r3_ASR_r7"},
     {{vs, r6, r6, ROR, r7}, true, vs, "vs r6 r6 ROR r7", "vs_r6_r6_ROR_r7"},
     {{le, r0, r0, LSL, r1}, true, le, "le r0 r0 LSL r1", "le_r0_r0_LSL_r1"},
     {{le, r7, r7, ROR, r1}, true, le, "le r7 r7 ROR r1", "le_r7_r7_ROR_r1"},
     {{ne, r6, r6, LSL, r0}, true, ne, "ne r6 r6 LSL r0", "ne_r6_r6_LSL_r0"},
     {{hi, r2, r2, LSL, r7}, true, hi, "hi r2 r2 LSL r7", "hi_r2_r2_LSL_r7"},
     {{lt, r0, r0, ASR, r0}, true, lt, "lt r0 r0 ASR r0", "lt_r0_r0_ASR_r0"},
     {{cc, r2, r2, ROR, r6}, true, cc, "cc r2 r2 ROR r6", "cc_r2_r2_ROR_r6"},
     {{pl, r5, r5, LSR, r2}, true, pl, "pl r5 r5 LSR r2", "pl_r5_r5_LSR_r2"},
     {{lt, r0, r0, ROR, r4}, true, lt, "lt r0 r0 ROR r4", "lt_r0_r0_ROR_r4"},
     {{ls, r6, r6, ASR, r1}, true, ls, "ls r6 r6 ASR r1", "ls_r6_r6_ASR_r1"},
     {{ne, r3, r3, LSR, r7}, true, ne, "ne r3 r3 LSR r7", "ne_r3_r3_LSR_r7"},
     {{vs, r0, r0, ROR, r4}, true, vs, "vs r0 r0 ROR r4", "vs_r0_r0_ROR_r4"},
     {{ge, r5, r5, ASR, r7}, true, ge, "ge r5 r5 ASR r7", "ge_r5_r5_ASR_r7"},
     {{pl, r5, r5, ROR, r2}, true, pl, "pl r5 r5 ROR r2", "pl_r5_r5_ROR_r2"},
     {{le, r1, r1, ROR, r5}, true, le, "le r1 r1 ROR r5", "le_r1_r1_ROR_r5"},
     {{ls, r4, r4, LSR, r1}, true, ls, "ls r4 r4 LSR r1", "ls_r4_r4_LSR_r1"},
     {{vc, r0, r0, LSL, r3}, true, vc, "vc r0 r0 LSL r3", "vc_r0_r0_LSL_r3"},
     {{vc, r0, r0, ASR, r3}, true, vc, "vc r0 r0 ASR r3", "vc_r0_r0_ASR_r3"},
     {{pl, r7, r7, ASR, r1}, true, pl, "pl r7 r7 ASR r1", "pl_r7_r7_ASR_r1"},
     {{cc, r0, r0, LSL, r1}, true, cc, "cc r0 r0 LSL r1", "cc_r0_r0_LSL_r1"},
     {{hi, r1, r1, LSL, r0}, true, hi, "hi r1 r1 LSL r0", "hi_r1_r1_LSL_r0"},
     {{gt, r6, r6, LSL, r2}, true, gt, "gt r6 r6 LSL r2", "gt_r6_r6_LSL_r2"},
     {{vc, r2, r2, LSR, r6}, true, vc, "vc r2 r2 LSR r6", "vc_r2_r2_LSR_r6"},
     {{vc, r5, r5, LSL, r5}, true, vc, "vc r5 r5 LSL r5", "vc_r5_r5_LSL_r5"},
     {{vc, r4, r4, LSR, r1}, true, vc, "vc r4 r4 LSR r1", "vc_r4_r4_LSR_r1"},
     {{lt, r4, r4, LSL, r6}, true, lt, "lt r4 r4 LSL r6", "lt_r4_r4_LSL_r6"},
     {{gt, r6, r6, ASR, r1}, true, gt, "gt r6 r6 ASR r1", "gt_r6_r6_ASR_r1"},
     {{hi, r1, r1, ASR, r1}, true, hi, "hi r1 r1 ASR r1", "hi_r1_r1_ASR_r1"},
     {{gt, r6, r6, ROR, r4}, true, gt, "gt r6 r6 ROR r4", "gt_r6_r6_ROR_r4"},
     {{gt, r3, r3, LSL, r3}, true, gt, "gt r3 r3 LSL r3", "gt_r3_r3_LSL_r3"},
     {{vs, r3, r3, ROR, r0}, true, vs, "vs r3 r3 ROR r0", "vs_r3_r3_ROR_r0"},
     {{lt, r4, r4, ROR, r1}, true, lt, "lt r4 r4 ROR r1", "lt_r4_r4_ROR_r1"},
     {{gt, r7, r7, LSL, r5}, true, gt, "gt r7 r7 LSL r5", "gt_r7_r7_LSL_r5"},
     {{hi, r0, r0, ASR, r5}, true, hi, "hi r0 r0 ASR r5", "hi_r0_r0_ASR_r5"},
     {{vc, r4, r4, ROR, r4}, true, vc, "vc r4 r4 ROR r4", "vc_r4_r4_ROR_r4"},
     {{gt, r5, r5, ASR, r0}, true, gt, "gt r5 r5 ASR r0", "gt_r5_r5_ASR_r0"},
     {{gt, r4, r4, LSR, r0}, true, gt, "gt r4 r4 LSR r0", "gt_r4_r4_LSR_r0"},
     {{gt, r3, r3, ASR, r3}, true, gt, "gt r3 r3 ASR r3", "gt_r3_r3_ASR_r3"},
     {{lt, r5, r5, ROR, r3}, true, lt, "lt r5 r5 ROR r3", "lt_r5_r5_ROR_r3"},
     {{cc, r7, r7, LSL, r1}, true, cc, "cc r7 r7 LSL r1", "cc_r7_r7_LSL_r1"},
     {{vc, r1, r1, LSR, r2}, true, vc, "vc r1 r1 LSR r2", "vc_r1_r1_LSR_r2"},
     {{pl, r3, r3, ASR, r5}, true, pl, "pl r3 r3 ASR r5", "pl_r3_r3_ASR_r5"},
     {{ls, r3, r3, LSL, r0}, true, ls, "ls r3 r3 LSL r0", "ls_r3_r3_LSL_r0"},
     {{vs, r5, r5, ROR, r4}, true, vs, "vs r5 r5 ROR r4", "vs_r5_r5_ROR_r4"},
     {{hi, r2, r2, LSL, r6}, true, hi, "hi r2 r2 LSL r6", "hi_r2_r2_LSL_r6"},
     {{cc, r5, r5, ROR, r0}, true, cc, "cc r5 r5 ROR r0", "cc_r5_r5_ROR_r0"},
     {{cs, r2, r2, ASR, r5}, true, cs, "cs r2 r2 ASR r5", "cs_r2_r2_ASR_r5"},
     {{ge, r1, r1, ROR, r3}, true, ge, "ge r1 r1 ROR r3", "ge_r1_r1_ROR_r3"},
     {{ne, r0, r0, ROR, r3}, true, ne, "ne r0 r0 ROR r3", "ne_r0_r0_ROR_r3"},
     {{cs, r7, r7, ASR, r6}, true, cs, "cs r7 r7 ASR r6", "cs_r7_r7_ASR_r6"},
     {{mi, r1, r1, LSR, r0}, true, mi, "mi r1 r1 LSR r0", "mi_r1_r1_LSR_r0"},
     {{mi, r5, r5, LSL, r1}, true, mi, "mi r5 r5 LSL r1", "mi_r5_r5_LSL_r1"},
     {{mi, r4, r4, LSR, r3}, true, mi, "mi r4 r4 LSR r3", "mi_r4_r4_LSR_r3"},
     {{pl, r3, r3, ASR, r7}, true, pl, "pl r3 r3 ASR r7", "pl_r3_r3_ASR_r7"},
     {{ls, r2, r2, ASR, r2}, true, ls, "ls r2 r2 ASR r2", "ls_r2_r2_ASR_r2"},
     {{eq, r7, r7, LSL, r3}, true, eq, "eq r7 r7 LSL r3", "eq_r7_r7_LSL_r3"},
     {{gt, r4, r4, LSL, r2}, true, gt, "gt r4 r4 LSL r2", "gt_r4_r4_LSL_r2"},
     {{ne, r6, r6, LSL, r6}, true, ne, "ne r6 r6 LSL r6", "ne_r6_r6_LSL_r6"},
     {{eq, r7, r7, LSL, r0}, true, eq, "eq r7 r7 LSL r0", "eq_r7_r7_LSL_r0"},
     {{le, r7, r7, LSL, r7}, true, le, "le r7 r7 LSL r7", "le_r7_r7_LSL_r7"},
     {{eq, r0, r0, ROR, r1}, true, eq, "eq r0 r0 ROR r1", "eq_r0_r0_ROR_r1"},
     {{hi, r2, r2, LSR, r6}, true, hi, "hi r2 r2 LSR r6", "hi_r2_r2_LSR_r6"},
     {{vs, r6, r6, LSL, r5}, true, vs, "vs r6 r6 LSL r5", "vs_r6_r6_LSL_r5"},
     {{gt, r5, r5, ASR, r1}, true, gt, "gt r5 r5 ASR r1", "gt_r5_r5_ASR_r1"},
     {{ne, r4, r4, ASR, r3}, true, ne, "ne r4 r4 ASR r3", "ne_r4_r4_ASR_r3"},
     {{lt, r6, r6, LSL, r3}, true, lt, "lt r6 r6 LSL r3", "lt_r6_r6_LSL_r3"},
     {{ls, r3, r3, ASR, r3}, true, ls, "ls r3 r3 ASR r3", "ls_r3_r3_ASR_r3"},
     {{le, r4, r4, LSL, r1}, true, le, "le r4 r4 LSL r1", "le_r4_r4_LSL_r1"},
     {{gt, r7, r7, LSL, r6}, true, gt, "gt r7 r7 LSL r6", "gt_r7_r7_LSL_r6"},
     {{vc, r2, r2, ROR, r1}, true, vc, "vc r2 r2 ROR r1", "vc_r2_r2_ROR_r1"},
     {{ne, r1, r1, ROR, r1}, true, ne, "ne r1 r1 ROR r1", "ne_r1_r1_ROR_r1"},
     {{cc, r3, r3, ASR, r0}, true, cc, "cc r3 r3 ASR r0", "cc_r3_r3_ASR_r0"},
     {{lt, r4, r4, LSR, r0}, true, lt, "lt r4 r4 LSR r0", "lt_r4_r4_LSR_r0"},
     {{cc, r3, r3, LSR, r7}, true, cc, "cc r3 r3 LSR r7", "cc_r3_r3_LSR_r7"},
     {{eq, r0, r0, LSR, r5}, true, eq, "eq r0 r0 LSR r5", "eq_r0_r0_LSR_r5"},
     {{lt, r5, r5, ROR, r5}, true, lt, "lt r5 r5 ROR r5", "lt_r5_r5_ROR_r5"},
     {{vc, r0, r0, LSL, r7}, true, vc, "vc r0 r0 LSL r7", "vc_r0_r0_LSL_r7"},
     {{cs, r7, r7, LSR, r5}, true, cs, "cs r7 r7 LSR r5", "cs_r7_r7_LSR_r5"},
     {{lt, r4, r4, ROR, r0}, true, lt, "lt r4 r4 ROR r0", "lt_r4_r4_ROR_r0"},
     {{vc, r4, r4, ROR, r3}, true, vc, "vc r4 r4 ROR r3", "vc_r4_r4_ROR_r3"},
     {{le, r6, r6, LSL, r6}, true, le, "le r6 r6 LSL r6", "le_r6_r6_LSL_r6"},
     {{cc, r4, r4, LSR, r7}, true, cc, "cc r4 r4 LSR r7", "cc_r4_r4_LSR_r7"},
     {{pl, r7, r7, ROR, r2}, true, pl, "pl r7 r7 ROR r2", "pl_r7_r7_ROR_r2"},
     {{hi, r0, r0, LSL, r6}, true, hi, "hi r0 r0 LSL r6", "hi_r0_r0_LSL_r6"},
     {{gt, r1, r1, LSR, r4}, true, gt, "gt r1 r1 LSR r4", "gt_r1_r1_LSR_r4"},
     {{vs, r2, r2, LSR, r1}, true, vs, "vs r2 r2 LSR r1", "vs_r2_r2_LSR_r1"},
     {{cc, r3, r3, ROR, r0}, true, cc, "cc r3 r3 ROR r0", "cc_r3_r3_ROR_r0"},
     {{ge, r6, r6, LSR, r5}, true, ge, "ge r6 r6 LSR r5", "ge_r6_r6_LSR_r5"},
     {{ne, r2, r2, ASR, r2}, true, ne, "ne r2 r2 ASR r2", "ne_r2_r2_ASR_r2"},
     {{lt, r5, r5, LSR, r7}, true, lt, "lt r5 r5 LSR r7", "lt_r5_r5_LSR_r7"},
     {{vs, r3, r3, LSL, r4}, true, vs, "vs r3 r3 LSL r4", "vs_r3_r3_LSL_r4"},
     {{lt, r7, r7, LSR, r5}, true, lt, "lt r7 r7 LSR r5", "lt_r7_r7_LSR_r5"},
     {{vc, r2, r2, LSL, r5}, true, vc, "vc r2 r2 LSL r5", "vc_r2_r2_LSL_r5"},
     {{le, r6, r6, ASR, r7}, true, le, "le r6 r6 ASR r7", "le_r6_r6_ASR_r7"},
     {{ne, r1, r1, LSL, r6}, true, ne, "ne r1 r1 LSL r6", "ne_r1_r1_LSL_r6"},
     {{le, r6, r6, ASR, r1}, true, le, "le r6 r6 ASR r1", "le_r6_r6_ASR_r1"},
     {{ls, r1, r1, ROR, r7}, true, ls, "ls r1 r1 ROR r7", "ls_r1_r1_ROR_r7"},
     {{pl, r0, r0, LSL, r0}, true, pl, "pl r0 r0 LSL r0", "pl_r0_r0_LSL_r0"},
     {{pl, r4, r4, ROR, r1}, true, pl, "pl r4 r4 ROR r1", "pl_r4_r4_ROR_r1"},
     {{ne, r7, r7, ASR, r3}, true, ne, "ne r7 r7 ASR r3", "ne_r7_r7_ASR_r3"},
     {{hi, r6, r6, LSR, r7}, true, hi, "hi r6 r6 LSR r7", "hi_r6_r6_LSR_r7"},
     {{eq, r5, r5, LSL, r4}, true, eq, "eq r5 r5 LSL r4", "eq_r5_r5_LSL_r4"},
     {{hi, r7, r7, LSR, r4}, true, hi, "hi r7 r7 LSR r4", "hi_r7_r7_LSR_r4"},
     {{ls, r3, r3, ASR, r0}, true, ls, "ls r3 r3 ASR r0", "ls_r3_r3_ASR_r0"},
     {{mi, r2, r2, ASR, r0}, true, mi, "mi r2 r2 ASR r0", "mi_r2_r2_ASR_r0"},
     {{gt, r7, r7, ROR, r3}, true, gt, "gt r7 r7 ROR r3", "gt_r7_r7_ROR_r3"},
     {{ge, r1, r1, ASR, r5}, true, ge, "ge r1 r1 ASR r5", "ge_r1_r1_ASR_r5"},
     {{vc, r3, r3, LSR, r3}, true, vc, "vc r3 r3 LSR r3", "vc_r3_r3_LSR_r3"},
     {{gt, r6, r6, LSR, r2}, true, gt, "gt r6 r6 LSR r2", "gt_r6_r6_LSR_r2"},
     {{vc, r6, r6, LSL, r2}, true, vc, "vc r6 r6 LSL r2", "vc_r6_r6_LSL_r2"},
     {{le, r7, r7, ASR, r1}, true, le, "le r7 r7 ASR r1", "le_r7_r7_ASR_r1"},
     {{pl, r1, r1, LSR, r7}, true, pl, "pl r1 r1 LSR r7", "pl_r1_r1_LSR_r7"},
     {{mi, r1, r1, ROR, r5}, true, mi, "mi r1 r1 ROR r5", "mi_r1_r1_ROR_r5"},
     {{eq, r6, r6, ASR, r1}, true, eq, "eq r6 r6 ASR r1", "eq_r6_r6_ASR_r1"},
     {{vs, r4, r4, LSR, r6}, true, vs, "vs r4 r4 LSR r6", "vs_r4_r4_LSR_r6"},
     {{ls, r0, r0, ASR, r0}, true, ls, "ls r0 r0 ASR r0", "ls_r0_r0_ASR_r0"},
     {{cc, r0, r0, LSL, r2}, true, cc, "cc r0 r0 LSL r2", "cc_r0_r0_LSL_r2"},
     {{cc, r3, r3, ROR, r7}, true, cc, "cc r3 r3 ROR r7", "cc_r3_r3_ROR_r7"},
     {{ge, r2, r2, ROR, r7}, true, ge, "ge r2 r2 ROR r7", "ge_r2_r2_ROR_r7"},
     {{mi, r7, r7, LSL, r4}, true, mi, "mi r7 r7 LSL r4", "mi_r7_r7_LSL_r4"},
     {{le, r4, r4, LSR, r5}, true, le, "le r4 r4 LSR r5", "le_r4_r4_LSR_r5"},
     {{cc, r5, r5, ASR, r3}, true, cc, "cc r5 r5 ASR r3", "cc_r5_r5_ASR_r3"},
     {{ls, r0, r0, ASR, r6}, true, ls, "ls r0 r0 ASR r6", "ls_r0_r0_ASR_r6"},
     {{ge, r4, r4, LSL, r1}, true, ge, "ge r4 r4 LSL r1", "ge_r4_r4_LSL_r1"},
     {{pl, r1, r1, LSL, r5}, true, pl, "pl r1 r1 LSL r5", "pl_r1_r1_LSL_r5"},
     {{ne, r2, r2, ASR, r5}, true, ne, "ne r2 r2 ASR r5", "ne_r2_r2_ASR_r5"},
     {{eq, r4, r4, ROR, r3}, true, eq, "eq r4 r4 ROR r3", "eq_r4_r4_ROR_r3"},
     {{mi, r2, r2, LSL, r3}, true, mi, "mi r2 r2 LSL r3", "mi_r2_r2_LSL_r3"},
     {{lt, r2, r2, ROR, r4}, true, lt, "lt r2 r2 ROR r4", "lt_r2_r2_ROR_r4"},
     {{ge, r7, r7, LSR, r2}, true, ge, "ge r7 r7 LSR r2", "ge_r7_r7_LSR_r2"},
     {{gt, r4, r4, ASR, r5}, true, gt, "gt r4 r4 ASR r5", "gt_r4_r4_ASR_r5"},
     {{le, r2, r2, LSL, r2}, true, le, "le r2 r2 LSL r2", "le_r2_r2_LSL_r2"},
     {{eq, r3, r3, LSR, r5}, true, eq, "eq r3 r3 LSR r5", "eq_r3_r3_LSR_r5"},
     {{ge, r7, r7, ASR, r1}, true, ge, "ge r7 r7 ASR r1", "ge_r7_r7_ASR_r1"},
     {{vc, r1, r1, LSL, r2}, true, vc, "vc r1 r1 LSL r2", "vc_r1_r1_LSL_r2"},
     {{cs, r2, r2, ASR, r3}, true, cs, "cs r2 r2 ASR r3", "cs_r2_r2_ASR_r3"},
     {{ge, r6, r6, LSL, r7}, true, ge, "ge r6 r6 LSL r7", "ge_r6_r6_LSL_r7"},
     {{hi, r5, r5, LSL, r0}, true, hi, "hi r5 r5 LSL r0", "hi_r5_r5_LSL_r0"},
     {{eq, r4, r4, LSR, r1}, true, eq, "eq r4 r4 LSR r1", "eq_r4_r4_LSR_r1"},
     {{gt, r5, r5, LSL, r1}, true, gt, "gt r5 r5 LSL r1", "gt_r5_r5_LSL_r1"},
     {{vs, r5, r5, LSL, r7}, true, vs, "vs r5 r5 LSL r7", "vs_r5_r5_LSL_r7"},
     {{ge, r7, r7, LSL, r2}, true, ge, "ge r7 r7 LSL r2", "ge_r7_r7_LSL_r2"},
     {{hi, r0, r0, ASR, r7}, true, hi, "hi r0 r0 ASR r7", "hi_r0_r0_ASR_r7"},
     {{cs, r3, r3, LSL, r3}, true, cs, "cs r3 r3 LSL r3", "cs_r3_r3_LSL_r3"},
     {{lt, r2, r2, LSL, r6}, true, lt, "lt r2 r2 LSL r6", "lt_r2_r2_LSL_r6"},
     {{pl, r1, r1, ASR, r4}, true, pl, "pl r1 r1 ASR r4", "pl_r1_r1_ASR_r4"},
     {{vc, r1, r1, ASR, r5}, true, vc, "vc r1 r1 ASR r5", "vc_r1_r1_ASR_r5"},
     {{lt, r4, r4, ROR, r5}, true, lt, "lt r4 r4 ROR r5", "lt_r4_r4_ROR_r5"},
     {{cc, r0, r0, ASR, r3}, true, cc, "cc r0 r0 ASR r3", "cc_r0_r0_ASR_r3"},
     {{vs, r4, r4, LSL, r5}, true, vs, "vs r4 r4 LSL r5", "vs_r4_r4_LSL_r5"},
     {{vs, r0, r0, LSL, r3}, true, vs, "vs r0 r0 LSL r3", "vs_r0_r0_LSL_r3"},
     {{hi, r1, r1, ROR, r7}, true, hi, "hi r1 r1 ROR r7", "hi_r1_r1_ROR_r7"},
     {{hi, r1, r1, LSL, r5}, true, hi, "hi r1 r1 LSL r5", "hi_r1_r1_LSL_r5"},
     {{le, r0, r0, LSR, r7}, true, le, "le r0 r0 LSR r7", "le_r0_r0_LSR_r7"},
     {{ls, r2, r2, ASR, r3}, true, ls, "ls r2 r2 ASR r3", "ls_r2_r2_ASR_r3"},
     {{mi, r3, r3, LSL, r3}, true, mi, "mi r3 r3 LSL r3", "mi_r3_r3_LSL_r3"},
     {{ne, r6, r6, ASR, r1}, true, ne, "ne r6 r6 ASR r1", "ne_r6_r6_ASR_r1"},
     {{le, r6, r6, LSL, r5}, true, le, "le r6 r6 LSL r5", "le_r6_r6_LSL_r5"},
     {{gt, r5, r5, LSR, r6}, true, gt, "gt r5 r5 LSR r6", "gt_r5_r5_LSR_r6"},
     {{vs, r5, r5, ASR, r1}, true, vs, "vs r5 r5 ASR r1", "vs_r5_r5_ASR_r1"},
     {{ne, r3, r3, LSL, r1}, true, ne, "ne r3 r3 LSL r1", "ne_r3_r3_LSL_r1"},
     {{lt, r2, r2, LSL, r4}, true, lt, "lt r2 r2 LSL r4", "lt_r2_r2_LSL_r4"},
     {{eq, r1, r1, ROR, r1}, true, eq, "eq r1 r1 ROR r1", "eq_r1_r1_ROR_r1"},
     {{cs, r1, r1, LSL, r6}, true, cs, "cs r1 r1 LSL r6", "cs_r1_r1_LSL_r6"},
     {{hi, r7, r7, ASR, r2}, true, hi, "hi r7 r7 ASR r2", "hi_r7_r7_ASR_r2"},
     {{mi, r4, r4, LSL, r3}, true, mi, "mi r4 r4 LSL r3", "mi_r4_r4_LSL_r3"},
     {{eq, r3, r3, LSL, r4}, true, eq, "eq r3 r3 LSL r4", "eq_r3_r3_LSL_r4"},
     {{ne, r6, r6, LSL, r3}, true, ne, "ne r6 r6 LSL r3", "ne_r6_r6_LSL_r3"},
     {{vs, r4, r4, LSR, r0}, true, vs, "vs r4 r4 LSR r0", "vs_r4_r4_LSR_r0"},
     {{eq, r6, r6, ASR, r4}, true, eq, "eq r6 r6 ASR r4", "eq_r6_r6_ASR_r4"},
     {{pl, r0, r0, ASR, r1}, true, pl, "pl r0 r0 ASR r1", "pl_r0_r0_ASR_r1"},
     {{cs, r7, r7, LSR, r3}, true, cs, "cs r7 r7 LSR r3", "cs_r7_r7_LSR_r3"},
     {{le, r2, r2, LSR, r2}, true, le, "le r2 r2 LSR r2", "le_r2_r2_LSR_r2"},
     {{ge, r3, r3, LSR, r5}, true, ge, "ge r3 r3 LSR r5", "ge_r3_r3_LSR_r5"},
     {{ls, r0, r0, ASR, r4}, true, ls, "ls r0 r0 ASR r4", "ls_r0_r0_ASR_r4"},
     {{lt, r6, r6, LSR, r1}, true, lt, "lt r6 r6 LSR r1", "lt_r6_r6_LSR_r1"},
     {{vc, r0, r0, ASR, r5}, true, vc, "vc r0 r0 ASR r5", "vc_r0_r0_ASR_r5"},
     {{lt, r2, r2, LSR, r0}, true, lt, "lt r2 r2 LSR r0", "lt_r2_r2_LSR_r0"},
     {{pl, r5, r5, ASR, r0}, true, pl, "pl r5 r5 ASR r0", "pl_r5_r5_ASR_r0"},
     {{pl, r1, r1, ROR, r6}, true, pl, "pl r1 r1 ROR r6", "pl_r1_r1_ROR_r6"},
     {{hi, r4, r4, ASR, r7}, true, hi, "hi r4 r4 ASR r7", "hi_r4_r4_ASR_r7"},
     {{le, r6, r6, ROR, r3}, true, le, "le r6 r6 ROR r3", "le_r6_r6_ROR_r3"},
     {{cc, r1, r1, LSL, r3}, true, cc, "cc r1 r1 LSL r3", "cc_r1_r1_LSL_r3"},
     {{vc, r1, r1, ROR, r1}, true, vc, "vc r1 r1 ROR r1", "vc_r1_r1_ROR_r1"},
     {{vc, r0, r0, ROR, r4}, true, vc, "vc r0 r0 ROR r4", "vc_r0_r0_ROR_r4"},
     {{ls, r3, r3, LSL, r5}, true, ls, "ls r3 r3 LSL r5", "ls_r3_r3_LSL_r5"},
     {{ls, r5, r5, ROR, r2}, true, ls, "ls r5 r5 ROR r2", "ls_r5_r5_ROR_r2"},
     {{lt, r0, r0, ROR, r5}, true, lt, "lt r0 r0 ROR r5", "lt_r0_r0_ROR_r5"},
     {{le, r7, r7, ROR, r4}, true, le, "le r7 r7 ROR r4", "le_r7_r7_ROR_r4"},
     {{eq, r1, r1, ROR, r0}, true, eq, "eq r1 r1 ROR r0", "eq_r1_r1_ROR_r0"},
     {{gt, r3, r3, LSL, r7}, true, gt, "gt r3 r3 LSL r7", "gt_r3_r3_LSL_r7"},
     {{vc, r0, r0, ASR, r6}, true, vc, "vc r0 r0 ASR r6", "vc_r0_r0_ASR_r6"},
     {{ge, r7, r7, LSL, r5}, true, ge, "ge r7 r7 LSL r5", "ge_r7_r7_LSL_r5"},
     {{pl, r5, r5, LSL, r4}, true, pl, "pl r5 r5 LSL r4", "pl_r5_r5_LSL_r4"},
     {{pl, r0, r0, ROR, r6}, true, pl, "pl r0 r0 ROR r6", "pl_r0_r0_ROR_r6"},
     {{hi, r1, r1, ROR, r1}, true, hi, "hi r1 r1 ROR r1", "hi_r1_r1_ROR_r1"},
     {{lt, r1, r1, ASR, r2}, true, lt, "lt r1 r1 ASR r2", "lt_r1_r1_ASR_r2"},
     {{cs, r4, r4, LSR, r6}, true, cs, "cs r4 r4 LSR r6", "cs_r4_r4_LSR_r6"},
     {{hi, r1, r1, ASR, r0}, true, hi, "hi r1 r1 ASR r0", "hi_r1_r1_ASR_r0"},
     {{le, r7, r7, ROR, r5}, true, le, "le r7 r7 ROR r5", "le_r7_r7_ROR_r5"},
     {{cs, r6, r6, LSL, r0}, true, cs, "cs r6 r6 LSL r0", "cs_r6_r6_LSL_r0"},
     {{hi, r3, r3, ROR, r3}, true, hi, "hi r3 r3 ROR r3", "hi_r3_r3_ROR_r3"},
     {{vs, r6, r6, LSR, r0}, true, vs, "vs r6 r6 LSR r0", "vs_r6_r6_LSR_r0"},
     {{ls, r5, r5, LSR, r6}, true, ls, "ls r5 r5 LSR r6", "ls_r5_r5_LSR_r6"},
     {{vs, r1, r1, ROR, r0}, true, vs, "vs r1 r1 ROR r0", "vs_r1_r1_ROR_r0"},
     {{ge, r3, r3, ASR, r4}, true, ge, "ge r3 r3 ASR r4", "ge_r3_r3_ASR_r4"},
     {{ne, r5, r5, ROR, r7}, true, ne, "ne r5 r5 ROR r7", "ne_r5_r5_ROR_r7"},
     {{ls, r5, r5, LSL, r2}, true, ls, "ls r5 r5 LSL r2", "ls_r5_r5_LSL_r2"},
     {{mi, r4, r4, ROR, r1}, true, mi, "mi r4 r4 ROR r1", "mi_r4_r4_ROR_r1"},
     {{cs, r6, r6, LSR, r6}, true, cs, "cs r6 r6 LSR r6", "cs_r6_r6_LSR_r6"},
     {{cs, r1, r1, ROR, r1}, true, cs, "cs r1 r1 ROR r1", "cs_r1_r1_ROR_r1"},
     {{ls, r5, r5, ROR, r3}, true, ls, "ls r5 r5 ROR r3", "ls_r5_r5_ROR_r3"},
     {{vs, r7, r7, LSR, r3}, true, vs, "vs r7 r7 LSR r3", "vs_r7_r7_LSR_r3"},
     {{le, r0, r0, ROR, r2}, true, le, "le r0 r0 ROR r2", "le_r0_r0_ROR_r2"},
     {{vc, r5, r5, LSR, r5}, true, vc, "vc r5 r5 LSR r5", "vc_r5_r5_LSR_r5"},
     {{vc, r4, r4, LSL, r3}, true, vc, "vc r4 r4 LSL r3", "vc_r4_r4_LSL_r3"},
     {{vc, r2, r2, LSR, r4}, true, vc, "vc r2 r2 LSR r4", "vc_r2_r2_LSR_r4"},
     {{ge, r5, r5, ROR, r2}, true, ge, "ge r5 r5 ROR r2", "ge_r5_r5_ROR_r2"},
     {{cc, r6, r6, ASR, r7}, true, cc, "cc r6 r6 ASR r7", "cc_r6_r6_ASR_r7"},
     {{hi, r6, r6, ASR, r5}, true, hi, "hi r6 r6 ASR r5", "hi_r6_r6_ASR_r5"},
     {{vs, r0, r0, LSR, r1}, true, vs, "vs r0 r0 LSR r1", "vs_r0_r0_LSR_r1"},
     {{ls, r4, r4, LSL, r0}, true, ls, "ls r4 r4 LSL r0", "ls_r4_r4_LSL_r0"},
     {{mi, r6, r6, ASR, r4}, true, mi, "mi r6 r6 ASR r4", "mi_r6_r6_ASR_r4"},
     {{le, r6, r6, LSL, r4}, true, le, "le r6 r6 LSL r4", "le_r6_r6_LSL_r4"},
     {{cs, r6, r6, ASR, r6}, true, cs, "cs r6 r6 ASR r6", "cs_r6_r6_ASR_r6"},
     {{mi, r6, r6, ROR, r7}, true, mi, "mi r6 r6 ROR r7", "mi_r6_r6_ROR_r7"},
     {{hi, r5, r5, LSL, r4}, true, hi, "hi r5 r5 LSL r4", "hi_r5_r5_LSL_r4"},
     {{hi, r6, r6, LSL, r7}, true, hi, "hi r6 r6 LSL r7", "hi_r6_r6_LSL_r7"},
     {{cc, r6, r6, ROR, r5}, true, cc, "cc r6 r6 ROR r5", "cc_r6_r6_ROR_r5"},
     {{ge, r0, r0, ASR, r2}, true, ge, "ge r0 r0 ASR r2", "ge_r0_r0_ASR_r2"},
     {{eq, r1, r1, LSR, r6}, true, eq, "eq r1 r1 LSR r6", "eq_r1_r1_LSR_r6"},
     {{le, r1, r1, LSR, r7}, true, le, "le r1 r1 LSR r7", "le_r1_r1_LSR_r7"},
     {{ls, r4, r4, LSR, r5}, true, ls, "ls r4 r4 LSR r5", "ls_r4_r4_LSR_r5"},
     {{hi, r6, r6, LSR, r4}, true, hi, "hi r6 r6 LSR r4", "hi_r6_r6_LSR_r4"},
     {{cc, r1, r1, ROR, r6}, true, cc, "cc r1 r1 ROR r6", "cc_r1_r1_ROR_r6"},
     {{mi, r3, r3, ROR, r4}, true, mi, "mi r3 r3 ROR r4", "mi_r3_r3_ROR_r4"},
     {{vc, r2, r2, LSL, r4}, true, vc, "vc r2 r2 LSL r4", "vc_r2_r2_LSL_r4"},
     {{hi, r3, r3, LSL, r0}, true, hi, "hi r3 r3 LSL r0", "hi_r3_r3_LSL_r0"},
     {{vc, r6, r6, ASR, r1}, true, vc, "vc r6 r6 ASR r1", "vc_r6_r6_ASR_r1"},
     {{pl, r6, r6, LSR, r1}, true, pl, "pl r6 r6 LSR r1", "pl_r6_r6_LSR_r1"},
     {{hi, r1, r1, ASR, r4}, true, hi, "hi r1 r1 ASR r4", "hi_r1_r1_ASR_r4"},
     {{ne, r3, r3, ASR, r0}, true, ne, "ne r3 r3 ASR r0", "ne_r3_r3_ASR_r0"},
     {{gt, r4, r4, ROR, r2}, true, gt, "gt r4 r4 ROR r2", "gt_r4_r4_ROR_r2"},
     {{ls, r7, r7, ASR, r2}, true, ls, "ls r7 r7 ASR r2", "ls_r7_r7_ASR_r2"},
     {{lt, r2, r2, LSL, r3}, true, lt, "lt r2 r2 LSL r3", "lt_r2_r2_LSL_r3"},
     {{pl, r4, r4, LSL, r7}, true, pl, "pl r4 r4 LSL r7", "pl_r4_r4_LSL_r7"},
     {{ge, r0, r0, ASR, r0}, true, ge, "ge r0 r0 ASR r0", "ge_r0_r0_ASR_r0"},
     {{gt, r4, r4, ASR, r4}, true, gt, "gt r4 r4 ASR r4", "gt_r4_r4_ASR_r4"},
     {{eq, r7, r7, ROR, r3}, true, eq, "eq r7 r7 ROR r3", "eq_r7_r7_ROR_r3"},
     {{ne, r2, r2, LSL, r0}, true, ne, "ne r2 r2 LSL r0", "ne_r2_r2_LSL_r0"},
     {{ge, r1, r1, LSR, r3}, true, ge, "ge r1 r1 LSR r3", "ge_r1_r1_LSR_r3"},
     {{ge, r4, r4, ROR, r5}, true, ge, "ge r4 r4 ROR r5", "ge_r4_r4_ROR_r5"},
     {{ne, r6, r6, LSR, r6}, true, ne, "ne r6 r6 LSR r6", "ne_r6_r6_LSR_r6"},
     {{le, r1, r1, ASR, r0}, true, le, "le r1 r1 ASR r0", "le_r1_r1_ASR_r0"},
     {{ge, r6, r6, LSR, r6}, true, ge, "ge r6 r6 LSR r6", "ge_r6_r6_LSR_r6"},
     {{pl, r5, r5, ROR, r6}, true, pl, "pl r5 r5 ROR r6", "pl_r5_r5_ROR_r6"},
     {{lt, r2, r2, LSL, r2}, true, lt, "lt r2 r2 LSL r2", "lt_r2_r2_LSL_r2"},
     {{hi, r7, r7, LSL, r0}, true, hi, "hi r7 r7 LSL r0", "hi_r7_r7_LSL_r0"},
     {{lt, r6, r6, ASR, r4}, true, lt, "lt r6 r6 ASR r4", "lt_r6_r6_ASR_r4"},
     {{ge, r3, r3, LSR, r6}, true, ge, "ge r3 r3 LSR r6", "ge_r3_r3_LSR_r6"},
     {{eq, r4, r4, ASR, r0}, true, eq, "eq r4 r4 ASR r0", "eq_r4_r4_ASR_r0"},
     {{ne, r7, r7, ROR, r0}, true, ne, "ne r7 r7 ROR r0", "ne_r7_r7_ROR_r0"},
     {{pl, r5, r5, LSR, r1}, true, pl, "pl r5 r5 LSR r1", "pl_r5_r5_LSR_r1"},
     {{pl, r3, r3, ROR, r2}, true, pl, "pl r3 r3 ROR r2", "pl_r3_r3_ROR_r2"},
     {{vc, r1, r1, ASR, r0}, true, vc, "vc r1 r1 ASR r0", "vc_r1_r1_ASR_r0"},
     {{eq, r0, r0, ASR, r6}, true, eq, "eq r0 r0 ASR r6", "eq_r0_r0_ASR_r6"},
     {{le, r3, r3, ROR, r4}, true, le, "le r3 r3 ROR r4", "le_r3_r3_ROR_r4"},
     {{lt, r7, r7, LSR, r2}, true, lt, "lt r7 r7 LSR r2", "lt_r7_r7_LSR_r2"},
     {{ls, r5, r5, ASR, r1}, true, ls, "ls r5 r5 ASR r1", "ls_r5_r5_ASR_r1"},
     {{cc, r6, r6, ROR, r6}, true, cc, "cc r6 r6 ROR r6", "cc_r6_r6_ROR_r6"},
     {{vs, r1, r1, ROR, r2}, true, vs, "vs r1 r1 ROR r2", "vs_r1_r1_ROR_r2"},
     {{mi, r5, r5, LSL, r6}, true, mi, "mi r5 r5 LSL r6", "mi_r5_r5_LSL_r6"},
     {{le, r1, r1, ROR, r1}, true, le, "le r1 r1 ROR r1", "le_r1_r1_ROR_r1"},
     {{vs, r2, r2, LSL, r4}, true, vs, "vs r2 r2 LSL r4", "vs_r2_r2_LSL_r4"},
     {{mi, r7, r7, LSR, r1}, true, mi, "mi r7 r7 LSR r1", "mi_r7_r7_LSR_r1"},
     {{vc, r5, r5, ASR, r3}, true, vc, "vc r5 r5 ASR r3", "vc_r5_r5_ASR_r3"},
     {{pl, r6, r6, ASR, r6}, true, pl, "pl r6 r6 ASR r6", "pl_r6_r6_ASR_r6"},
     {{pl, r6, r6, ROR, r5}, true, pl, "pl r6 r6 ROR r5", "pl_r6_r6_ROR_r5"},
     {{cs, r6, r6, LSL, r4}, true, cs, "cs r6 r6 LSL r4", "cs_r6_r6_LSL_r4"},
     {{mi, r5, r5, LSR, r3}, true, mi, "mi r5 r5 LSR r3", "mi_r5_r5_LSR_r3"},
     {{lt, r5, r5, ROR, r0}, true, lt, "lt r5 r5 ROR r0", "lt_r5_r5_ROR_r0"},
     {{gt, r0, r0, LSR, r1}, true, gt, "gt r0 r0 LSR r1", "gt_r0_r0_LSR_r1"},
     {{cs, r7, r7, LSL, r6}, true, cs, "cs r7 r7 LSL r6", "cs_r7_r7_LSL_r6"},
     {{cs, r5, r5, LSL, r1}, true, cs, "cs r5 r5 LSL r1", "cs_r5_r5_LSL_r1"},
     {{ge, r5, r5, ROR, r4}, true, ge, "ge r5 r5 ROR r4", "ge_r5_r5_ROR_r4"},
     {{le, r2, r2, LSR, r5}, true, le, "le r2 r2 LSR r5", "le_r2_r2_LSR_r5"},
     {{lt, r1, r1, LSL, r1}, true, lt, "lt r1 r1 LSL r1", "lt_r1_r1_LSL_r1"},
     {{hi, r7, r7, LSR, r6}, true, hi, "hi r7 r7 LSR r6", "hi_r7_r7_LSR_r6"},
     {{cs, r5, r5, LSR, r3}, true, cs, "cs r5 r5 LSR r3", "cs_r5_r5_LSR_r3"},
     {{cc, r2, r2, ROR, r7}, true, cc, "cc r2 r2 ROR r7", "cc_r2_r2_ROR_r7"},
     {{gt, r1, r1, ASR, r3}, true, gt, "gt r1 r1 ASR r3", "gt_r1_r1_ASR_r3"},
     {{le, r2, r2, ASR, r2}, true, le, "le r2 r2 ASR r2", "le_r2_r2_ASR_r2"},
     {{pl, r0, r0, ASR, r2}, true, pl, "pl r0 r0 ASR r2", "pl_r0_r0_ASR_r2"},
     {{ls, r2, r2, LSR, r0}, true, ls, "ls r2 r2 LSR r0", "ls_r2_r2_LSR_r0"},
     {{mi, r6, r6, LSR, r2}, true, mi, "mi r6 r6 LSR r2", "mi_r6_r6_LSR_r2"},
     {{vs, r0, r0, ASR, r5}, true, vs, "vs r0 r0 ASR r5", "vs_r0_r0_ASR_r5"},
     {{lt, r4, r4, LSL, r2}, true, lt, "lt r4 r4 LSL r2", "lt_r4_r4_LSL_r2"},
     {{le, r3, r3, LSR, r7}, true, le, "le r3 r3 LSR r7", "le_r3_r3_LSR_r7"},
     {{vs, r1, r1, ROR, r1}, true, vs, "vs r1 r1 ROR r1", "vs_r1_r1_ROR_r1"},
     {{gt, r5, r5, ASR, r3}, true, gt, "gt r5 r5 ASR r3", "gt_r5_r5_ASR_r3"},
     {{le, r2, r2, ROR, r5}, true, le, "le r2 r2 ROR r5", "le_r2_r2_ROR_r5"},
     {{le, r2, r2, ASR, r4}, true, le, "le r2 r2 ASR r4", "le_r2_r2_ASR_r4"},
     {{ls, r1, r1, LSL, r6}, true, ls, "ls r1 r1 LSL r6", "ls_r1_r1_LSL_r6"},
     {{cc, r6, r6, LSR, r3}, true, cc, "cc r6 r6 LSR r3", "cc_r6_r6_LSR_r3"},
     {{ne, r5, r5, LSL, r0}, true, ne, "ne r5 r5 LSL r0", "ne_r5_r5_LSL_r0"},
     {{mi, r1, r1, LSL, r2}, true, mi, "mi r1 r1 LSL r2", "mi_r1_r1_LSL_r2"},
     {{ne, r2, r2, ASR, r3}, true, ne, "ne r2 r2 ASR r3", "ne_r2_r2_ASR_r3"},
     {{hi, r1, r1, ROR, r3}, true, hi, "hi r1 r1 ROR r3", "hi_r1_r1_ROR_r3"},
     {{lt, r7, r7, ROR, r0}, true, lt, "lt r7 r7 ROR r0", "lt_r7_r7_ROR_r0"},
     {{pl, r4, r4, LSL, r6}, true, pl, "pl r4 r4 LSL r6", "pl_r4_r4_LSL_r6"},
     {{mi, r0, r0, ASR, r4}, true, mi, "mi r0 r0 ASR r4", "mi_r0_r0_ASR_r4"},
     {{cc, r1, r1, ROR, r1}, true, cc, "cc r1 r1 ROR r1", "cc_r1_r1_ROR_r1"},
     {{vc, r1, r1, LSL, r3}, true, vc, "vc r1 r1 LSL r3", "vc_r1_r1_LSL_r3"},
     {{pl, r1, r1, ROR, r2}, true, pl, "pl r1 r1 ROR r2", "pl_r1_r1_ROR_r2"},
     {{mi, r3, r3, ASR, r1}, true, mi, "mi r3 r3 ASR r1", "mi_r3_r3_ASR_r1"},
     {{vs, r5, r5, ROR, r2}, true, vs, "vs r5 r5 ROR r2", "vs_r5_r5_ROR_r2"},
     {{mi, r0, r0, ROR, r2}, true, mi, "mi r0 r0 ROR r2", "mi_r0_r0_ROR_r2"},
     {{ne, r7, r7, LSR, r1}, true, ne, "ne r7 r7 LSR r1", "ne_r7_r7_LSR_r1"},
     {{ne, r0, r0, LSL, r2}, true, ne, "ne r0 r0 LSL r2", "ne_r0_r0_LSL_r2"},
     {{gt, r5, r5, ROR, r4}, true, gt, "gt r5 r5 ROR r4", "gt_r5_r5_ROR_r4"},
     {{ls, r4, r4, ROR, r2}, true, ls, "ls r4 r4 ROR r2", "ls_r4_r4_ROR_r2"},
     {{gt, r5, r5, LSL, r0}, true, gt, "gt r5 r5 LSL r0", "gt_r5_r5_LSL_r0"},
     {{pl, r0, r0, LSL, r7}, true, pl, "pl r0 r0 LSL r7", "pl_r0_r0_LSL_r7"},
     {{vc, r3, r3, ROR, r2}, true, vc, "vc r3 r3 ROR r2", "vc_r3_r3_ROR_r2"},
     {{vs, r4, r4, LSL, r0}, true, vs, "vs r4 r4 LSL r0", "vs_r4_r4_LSL_r0"},
     {{vc, r6, r6, ROR, r0}, true, vc, "vc r6 r6 ROR r0", "vc_r6_r6_ROR_r0"},
     {{cc, r7, r7, LSR, r4}, true, cc, "cc r7 r7 LSR r4", "cc_r7_r7_LSR_r4"},
     {{cs, r0, r0, ASR, r7}, true, cs, "cs r0 r0 ASR r7", "cs_r0_r0_ASR_r7"},
     {{lt, r3, r3, ASR, r5}, true, lt, "lt r3 r3 ASR r5", "lt_r3_r3_ASR_r5"},
     {{cs, r6, r6, ROR, r4}, true, cs, "cs r6 r6 ROR r4", "cs_r6_r6_ROR_r4"},
     {{vs, r7, r7, ASR, r2}, true, vs, "vs r7 r7 ASR r2", "vs_r7_r7_ASR_r2"},
     {{ls, r4, r4, LSL, r2}, true, ls, "ls r4 r4 LSL r2", "ls_r4_r4_LSL_r2"},
     {{cc, r6, r6, LSL, r4}, true, cc, "cc r6 r6 LSL r4", "cc_r6_r6_LSL_r4"},
     {{ne, r7, r7, LSR, r3}, true, ne, "ne r7 r7 LSR r3", "ne_r7_r7_LSR_r3"},
     {{lt, r1, r1, LSR, r0}, true, lt, "lt r1 r1 LSR r0", "lt_r1_r1_LSR_r0"},
     {{ne, r6, r6, LSR, r1}, true, ne, "ne r6 r6 LSR r1", "ne_r6_r6_LSR_r1"},
     {{mi, r4, r4, ROR, r4}, true, mi, "mi r4 r4 ROR r4", "mi_r4_r4_ROR_r4"},
     {{eq, r4, r4, ASR, r1}, true, eq, "eq r4 r4 ASR r1", "eq_r4_r4_ASR_r1"},
     {{vc, r0, r0, LSL, r4}, true, vc, "vc r0 r0 LSL r4", "vc_r0_r0_LSL_r4"},
     {{ls, r3, r3, ROR, r6}, true, ls, "ls r3 r3 ROR r6", "ls_r3_r3_ROR_r6"},
     {{pl, r3, r3, LSL, r1}, true, pl, "pl r3 r3 LSL r1", "pl_r3_r3_LSL_r1"},
     {{mi, r4, r4, ASR, r6}, true, mi, "mi r4 r4 ASR r6", "mi_r4_r4_ASR_r6"},
     {{ne, r5, r5, ROR, r1}, true, ne, "ne r5 r5 ROR r1", "ne_r5_r5_ROR_r1"},
     {{le, r4, r4, ASR, r5}, true, le, "le r4 r4 ASR r5", "le_r4_r4_ASR_r5"},
     {{gt, r0, r0, ASR, r1}, true, gt, "gt r0 r0 ASR r1", "gt_r0_r0_ASR_r1"},
     {{cc, r3, r3, LSR, r0}, true, cc, "cc r3 r3 LSR r0", "cc_r3_r3_LSR_r0"},
     {{ge, r1, r1, LSL, r5}, true, ge, "ge r1 r1 LSL r5", "ge_r1_r1_LSL_r5"},
     {{gt, r4, r4, LSR, r4}, true, gt, "gt r4 r4 LSR r4", "gt_r4_r4_LSR_r4"},
     {{mi, r6, r6, ROR, r4}, true, mi, "mi r6 r6 ROR r4", "mi_r6_r6_ROR_r4"},
     {{pl, r4, r4, LSR, r7}, true, pl, "pl r4 r4 LSR r7", "pl_r4_r4_LSR_r7"},
     {{hi, r7, r7, ASR, r6}, true, hi, "hi r7 r7 ASR r6", "hi_r7_r7_ASR_r6"},
     {{vc, r1, r1, ROR, r4}, true, vc, "vc r1 r1 ROR r4", "vc_r1_r1_ROR_r4"},
     {{le, r4, r4, LSR, r7}, true, le, "le r4 r4 LSR r7", "le_r4_r4_LSR_r7"},
     {{le, r2, r2, ASR, r6}, true, le, "le r2 r2 ASR r6", "le_r2_r2_ASR_r6"},
     {{le, r4, r4, LSL, r2}, true, le, "le r4 r4 LSL r2", "le_r4_r4_LSL_r2"},
     {{eq, r7, r7, ASR, r4}, true, eq, "eq r7 r7 ASR r4", "eq_r7_r7_ASR_r4"},
     {{ls, r2, r2, LSL, r6}, true, ls, "ls r2 r2 LSL r6", "ls_r2_r2_LSL_r6"},
     {{pl, r3, r3, LSL, r6}, true, pl, "pl r3 r3 LSL r6", "pl_r3_r3_LSL_r6"},
     {{cc, r7, r7, LSR, r5}, true, cc, "cc r7 r7 LSR r5", "cc_r7_r7_LSR_r5"},
     {{cc, r1, r1, LSL, r7}, true, cc, "cc r1 r1 LSL r7", "cc_r1_r1_LSL_r7"},
     {{gt, r5, r5, LSR, r4}, true, gt, "gt r5 r5 LSR r4", "gt_r5_r5_LSR_r4"},
     {{ne, r2, r2, LSL, r4}, true, ne, "ne r2 r2 LSL r4", "ne_r2_r2_LSL_r4"},
     {{ge, r2, r2, ASR, r7}, true, ge, "ge r2 r2 ASR r7", "ge_r2_r2_ASR_r7"},
     {{gt, r2, r2, ROR, r7}, true, gt, "gt r2 r2 ROR r7", "gt_r2_r2_ROR_r7"},
     {{gt, r6, r6, ROR, r5}, true, gt, "gt r6 r6 ROR r5", "gt_r6_r6_ROR_r5"},
     {{lt, r4, r4, ASR, r3}, true, lt, "lt r4 r4 ASR r3", "lt_r4_r4_ASR_r3"},
     {{vc, r1, r1, LSR, r7}, true, vc, "vc r1 r1 LSR r7", "vc_r1_r1_LSR_r7"},
     {{ne, r2, r2, LSR, r7}, true, ne, "ne r2 r2 LSR r7", "ne_r2_r2_LSR_r7"},
     {{ls, r0, r0, LSR, r5}, true, ls, "ls r0 r0 LSR r5", "ls_r0_r0_LSR_r5"},
     {{cs, r3, r3, ASR, r1}, true, cs, "cs r3 r3 ASR r1", "cs_r3_r3_ASR_r1"},
     {{gt, r2, r2, ROR, r1}, true, gt, "gt r2 r2 ROR r1", "gt_r2_r2_ROR_r1"},
     {{eq, r0, r0, LSL, r3}, true, eq, "eq r0 r0 LSL r3", "eq_r0_r0_LSL_r3"},
     {{ls, r5, r5, ASR, r5}, true, ls, "ls r5 r5 ASR r5", "ls_r5_r5_ASR_r5"},
     {{cs, r5, r5, ROR, r5}, true, cs, "cs r5 r5 ROR r5", "cs_r5_r5_ROR_r5"},
     {{gt, r3, r3, ROR, r7}, true, gt, "gt r3 r3 ROR r7", "gt_r3_r3_ROR_r7"},
     {{vc, r3, r3, LSL, r6}, true, vc, "vc r3 r3 LSL r6", "vc_r3_r3_LSL_r6"},
     {{ge, r3, r3, LSR, r1}, true, ge, "ge r3 r3 LSR r1", "ge_r3_r3_LSR_r1"},
     {{cc, r2, r2, ROR, r3}, true, cc, "cc r2 r2 ROR r3", "cc_r2_r2_ROR_r3"},
     {{mi, r5, r5, ROR, r7}, true, mi, "mi r5 r5 ROR r7", "mi_r5_r5_ROR_r7"},
     {{eq, r6, r6, ASR, r3}, true, eq, "eq r6 r6 ASR r3", "eq_r6_r6_ASR_r3"},
     {{ne, r2, r2, LSL, r1}, true, ne, "ne r2 r2 LSL r1", "ne_r2_r2_LSL_r1"},
     {{mi, r5, r5, ROR, r6}, true, mi, "mi r5 r5 ROR r6", "mi_r5_r5_ROR_r6"},
     {{vc, r2, r2, ROR, r2}, true, vc, "vc r2 r2 ROR r2", "vc_r2_r2_ROR_r2"},
     {{pl, r7, r7, LSR, r1}, true, pl, "pl r7 r7 LSR r1", "pl_r7_r7_LSR_r1"},
     {{eq, r7, r7, ROR, r4}, true, eq, "eq r7 r7 ROR r4", "eq_r7_r7_ROR_r4"},
     {{eq, r1, r1, ASR, r4}, true, eq, "eq r1 r1 ASR r4", "eq_r1_r1_ASR_r4"},
     {{mi, r5, r5, LSL, r2}, true, mi, "mi r5 r5 LSL r2", "mi_r5_r5_LSL_r2"},
     {{pl, r0, r0, LSL, r4}, true, pl, "pl r0 r0 LSL r4", "pl_r0_r0_LSL_r4"},
     {{hi, r4, r4, LSR, r2}, true, hi, "hi r4 r4 LSR r2", "hi_r4_r4_LSR_r2"},
     {{lt, r5, r5, LSL, r6}, true, lt, "lt r5 r5 LSL r6", "lt_r5_r5_LSL_r6"},
     {{lt, r6, r6, ASR, r5}, true, lt, "lt r6 r6 ASR r5", "lt_r6_r6_ASR_r5"},
     {{eq, r1, r1, ASR, r0}, true, eq, "eq r1 r1 ASR r0", "eq_r1_r1_ASR_r0"},
     {{mi, r6, r6, ASR, r3}, true, mi, "mi r6 r6 ASR r3", "mi_r6_r6_ASR_r3"},
     {{lt, r0, r0, LSR, r4}, true, lt, "lt r0 r0 LSR r4", "lt_r0_r0_LSR_r4"},
     {{lt, r5, r5, ROR, r4}, true, lt, "lt r5 r5 ROR r4", "lt_r5_r5_ROR_r4"},
     {{ne, r1, r1, ASR, r7}, true, ne, "ne r1 r1 ASR r7", "ne_r1_r1_ASR_r7"},
     {{gt, r3, r3, LSL, r4}, true, gt, "gt r3 r3 LSL r4", "gt_r3_r3_LSL_r4"},
     {{le, r3, r3, ROR, r6}, true, le, "le r3 r3 ROR r6", "le_r3_r3_ROR_r6"},
     {{ne, r5, r5, ASR, r7}, true, ne, "ne r5 r5 ASR r7", "ne_r5_r5_ASR_r7"},
     {{hi, r5, r5, ROR, r6}, true, hi, "hi r5 r5 ROR r6", "hi_r5_r5_ROR_r6"},
     {{vs, r3, r3, LSR, r2}, true, vs, "vs r3 r3 LSR r2", "vs_r3_r3_LSR_r2"},
     {{ls, r1, r1, LSR, r1}, true, ls, "ls r1 r1 LSR r1", "ls_r1_r1_LSR_r1"},
     {{ls, r4, r4, LSR, r2}, true, ls, "ls r4 r4 LSR r2", "ls_r4_r4_LSR_r2"},
     {{gt, r7, r7, LSL, r1}, true, gt, "gt r7 r7 LSL r1", "gt_r7_r7_LSL_r1"},
     {{ge, r0, r0, ROR, r3}, true, ge, "ge r0 r0 ROR r3", "ge_r0_r0_ROR_r3"},
     {{gt, r7, r7, ASR, r4}, true, gt, "gt r7 r7 ASR r4", "gt_r7_r7_ASR_r4"},
     {{ne, r0, r0, LSL, r7}, true, ne, "ne r0 r0 LSL r7", "ne_r0_r0_LSL_r7"},
     {{ge, r2, r2, ROR, r1}, true, ge, "ge r2 r2 ROR r1", "ge_r2_r2_ROR_r1"},
     {{ge, r5, r5, LSR, r3}, true, ge, "ge r5 r5 LSR r3", "ge_r5_r5_LSR_r3"},
     {{ls, r2, r2, ASR, r0}, true, ls, "ls r2 r2 ASR r0", "ls_r2_r2_ASR_r0"},
     {{le, r5, r5, ASR, r7}, true, le, "le r5 r5 ASR r7", "le_r5_r5_ASR_r7"},
     {{le, r0, r0, LSR, r6}, true, le, "le r0 r0 LSR r6", "le_r0_r0_LSR_r6"},
     {{ne, r6, r6, ROR, r1}, true, ne, "ne r6 r6 ROR r1", "ne_r6_r6_ROR_r1"},
     {{vs, r2, r2, ASR, r0}, true, vs, "vs r2 r2 ASR r0", "vs_r2_r2_ASR_r0"},
     {{mi, r0, r0, LSR, r1}, true, mi, "mi r0 r0 LSR r1", "mi_r0_r0_LSR_r1"},
     {{cs, r1, r1, LSL, r5}, true, cs, "cs r1 r1 LSL r5", "cs_r1_r1_LSL_r5"},
     {{vc, r3, r3, ROR, r7}, true, vc, "vc r3 r3 ROR r7", "vc_r3_r3_ROR_r7"},
     {{gt, r6, r6, ASR, r0}, true, gt, "gt r6 r6 ASR r0", "gt_r6_r6_ASR_r0"},
     {{le, r2, r2, LSR, r3}, true, le, "le r2 r2 LSR r3", "le_r2_r2_LSR_r3"},
     {{lt, r4, r4, LSR, r3}, true, lt, "lt r4 r4 LSR r3", "lt_r4_r4_LSR_r3"},
     {{gt, r1, r1, ROR, r6}, true, gt, "gt r1 r1 ROR r6", "gt_r1_r1_ROR_r6"},
     {{ls, r6, r6, LSR, r4}, true, ls, "ls r6 r6 LSR r4", "ls_r6_r6_LSR_r4"},
     {{vc, r5, r5, ROR, r0}, true, vc, "vc r5 r5 ROR r0", "vc_r5_r5_ROR_r0"},
     {{ge, r2, r2, LSR, r2}, true, ge, "ge r2 r2 LSR r2", "ge_r2_r2_LSR_r2"},
     {{ls, r0, r0, LSR, r3}, true, ls, "ls r0 r0 LSR r3", "ls_r0_r0_LSR_r3"},
     {{gt, r6, r6, ASR, r6}, true, gt, "gt r6 r6 ASR r6", "gt_r6_r6_ASR_r6"},
     {{ne, r0, r0, LSL, r3}, true, ne, "ne r0 r0 LSL r3", "ne_r0_r0_LSL_r3"},
     {{mi, r0, r0, ASR, r1}, true, mi, "mi r0 r0 ASR r1", "mi_r0_r0_ASR_r1"},
     {{ge, r6, r6, ASR, r6}, true, ge, "ge r6 r6 ASR r6", "ge_r6_r6_ASR_r6"},
     {{pl, r2, r2, ASR, r2}, true, pl, "pl r2 r2 ASR r2", "pl_r2_r2_ASR_r2"},
     {{eq, r5, r5, LSL, r2}, true, eq, "eq r5 r5 LSL r2", "eq_r5_r5_LSL_r2"},
     {{vc, r2, r2, LSL, r3}, true, vc, "vc r2 r2 LSL r3", "vc_r2_r2_LSL_r3"},
     {{ls, r1, r1, LSL, r2}, true, ls, "ls r1 r1 LSL r2", "ls_r1_r1_LSL_r2"},
     {{vc, r3, r3, LSR, r0}, true, vc, "vc r3 r3 LSR r0", "vc_r3_r3_LSR_r0"},
     {{vc, r4, r4, ASR, r5}, true, vc, "vc r4 r4 ASR r5", "vc_r4_r4_ASR_r5"},
     {{ls, r2, r2, ASR, r1}, true, ls, "ls r2 r2 ASR r1", "ls_r2_r2_ASR_r1"},
     {{ls, r3, r3, LSL, r6}, true, ls, "ls r3 r3 LSL r6", "ls_r3_r3_LSL_r6"},
     {{vs, r6, r6, ROR, r5}, true, vs, "vs r6 r6 ROR r5", "vs_r6_r6_ROR_r5"},
     {{eq, r2, r2, LSR, r6}, true, eq, "eq r2 r2 LSR r6", "eq_r2_r2_LSR_r6"},
     {{hi, r3, r3, LSR, r5}, true, hi, "hi r3 r3 LSR r5", "hi_r3_r3_LSR_r5"},
     {{eq, r2, r2, ASR, r6}, true, eq, "eq r2 r2 ASR r6", "eq_r2_r2_ASR_r6"},
     {{mi, r2, r2, ASR, r4}, true, mi, "mi r2 r2 ASR r4", "mi_r2_r2_ASR_r4"},
     {{le, r6, r6, ROR, r6}, true, le, "le r6 r6 ROR r6", "le_r6_r6_ROR_r6"},
     {{eq, r0, r0, ROR, r7}, true, eq, "eq r0 r0 ROR r7", "eq_r0_r0_ROR_r7"},
     {{vs, r4, r4, LSL, r2}, true, vs, "vs r4 r4 LSL r2", "vs_r4_r4_LSL_r2"},
     {{vc, r2, r2, LSR, r5}, true, vc, "vc r2 r2 LSR r5", "vc_r2_r2_LSR_r5"},
     {{cc, r0, r0, ROR, r1}, true, cc, "cc r0 r0 ROR r1", "cc_r0_r0_ROR_r1"},
     {{lt, r5, r5, ASR, r0}, true, lt, "lt r5 r5 ASR r0", "lt_r5_r5_ASR_r0"},
     {{mi, r0, r0, LSL, r5}, true, mi, "mi r0 r0 LSL r5", "mi_r0_r0_LSL_r5"},
     {{eq, r6, r6, LSR, r1}, true, eq, "eq r6 r6 LSR r1", "eq_r6_r6_LSR_r1"},
     {{gt, r3, r3, ROR, r3}, true, gt, "gt r3 r3 ROR r3", "gt_r3_r3_ROR_r3"},
     {{gt, r6, r6, LSR, r7}, true, gt, "gt r6 r6 LSR r7", "gt_r6_r6_LSR_r7"},
     {{ne, r3, r3, LSL, r7}, true, ne, "ne r3 r3 LSL r7", "ne_r3_r3_LSL_r7"},
     {{cs, r2, r2, ASR, r4}, true, cs, "cs r2 r2 ASR r4", "cs_r2_r2_ASR_r4"},
     {{ge, r6, r6, ASR, r7}, true, ge, "ge r6 r6 ASR r7", "ge_r6_r6_ASR_r7"},
     {{hi, r7, r7, ASR, r0}, true, hi, "hi r7 r7 ASR r0", "hi_r7_r7_ASR_r0"},
     {{le, r2, r2, ASR, r1}, true, le, "le r2 r2 ASR r1", "le_r2_r2_ASR_r1"},
     {{vs, r6, r6, LSR, r5}, true, vs, "vs r6 r6 LSR r5", "vs_r6_r6_LSR_r5"},
     {{cs, r3, r3, ROR, r5}, true, cs, "cs r3 r3 ROR r5", "cs_r3_r3_ROR_r5"},
     {{le, r7, r7, LSR, r3}, true, le, "le r7 r7 LSR r3", "le_r7_r7_LSR_r3"},
     {{vc, r2, r2, ASR, r0}, true, vc, "vc r2 r2 ASR r0", "vc_r2_r2_ASR_r0"},
     {{ne, r7, r7, LSL, r3}, true, ne, "ne r7 r7 LSL r3", "ne_r7_r7_LSL_r3"},
     {{vs, r6, r6, ASR, r1}, true, vs, "vs r6 r6 ASR r1", "vs_r6_r6_ASR_r1"},
     {{cs, r4, r4, LSL, r0}, true, cs, "cs r4 r4 LSL r0", "cs_r4_r4_LSL_r0"},
     {{hi, r3, r3, ASR, r7}, true, hi, "hi r3 r3 ASR r7", "hi_r3_r3_ASR_r7"},
     {{eq, r5, r5, ASR, r2}, true, eq, "eq r5 r5 ASR r2", "eq_r5_r5_ASR_r2"},
     {{ge, r0, r0, ASR, r1}, true, ge, "ge r0 r0 ASR r1", "ge_r0_r0_ASR_r1"},
     {{cs, r1, r1, LSR, r0}, true, cs, "cs r1 r1 LSR r0", "cs_r1_r1_LSR_r0"},
     {{ne, r7, r7, LSL, r1}, true, ne, "ne r7 r7 LSL r1", "ne_r7_r7_LSL_r1"},
     {{cc, r1, r1, LSR, r0}, true, cc, "cc r1 r1 LSR r0", "cc_r1_r1_LSR_r0"},
     {{cs, r0, r0, LSL, r3}, true, cs, "cs r0 r0 LSL r3", "cs_r0_r0_LSL_r3"},
     {{hi, r3, r3, ASR, r6}, true, hi, "hi r3 r3 ASR r6", "hi_r3_r3_ASR_r6"},
     {{cs, r4, r4, ROR, r1}, true, cs, "cs r4 r4 ROR r1", "cs_r4_r4_ROR_r1"},
     {{eq, r0, r0, LSL, r6}, true, eq, "eq r0 r0 LSL r6", "eq_r0_r0_LSL_r6"},
     {{vc, r3, r3, ROR, r3}, true, vc, "vc r3 r3 ROR r3", "vc_r3_r3_ROR_r3"},
     {{le, r0, r0, ASR, r0}, true, le, "le r0 r0 ASR r0", "le_r0_r0_ASR_r0"},
     {{vs, r4, r4, LSR, r5}, true, vs, "vs r4 r4 LSR r5", "vs_r4_r4_LSR_r5"},
     {{ne, r4, r4, ROR, r2}, true, ne, "ne r4 r4 ROR r2", "ne_r4_r4_ROR_r2"},
     {{gt, r5, r5, ASR, r6}, true, gt, "gt r5 r5 ASR r6", "gt_r5_r5_ASR_r6"},
     {{hi, r7, r7, LSR, r0}, true, hi, "hi r7 r7 LSR r0", "hi_r7_r7_LSR_r0"},
     {{vc, r1, r1, ROR, r3}, true, vc, "vc r1 r1 ROR r3", "vc_r1_r1_ROR_r3"},
     {{pl, r0, r0, ASR, r4}, true, pl, "pl r0 r0 ASR r4", "pl_r0_r0_ASR_r4"},
     {{mi, r1, r1, LSL, r3}, true, mi, "mi r1 r1 LSL r3", "mi_r1_r1_LSL_r3"},
     {{pl, r2, r2, ASR, r0}, true, pl, "pl r2 r2 ASR r0", "pl_r2_r2_ASR_r0"},
     {{mi, r2, r2, ROR, r5}, true, mi, "mi r2 r2 ROR r5", "mi_r2_r2_ROR_r5"},
     {{vs, r1, r1, ROR, r4}, true, vs, "vs r1 r1 ROR r4", "vs_r1_r1_ROR_r4"},
     {{hi, r6, r6, ROR, r0}, true, hi, "hi r6 r6 ROR r0", "hi_r6_r6_ROR_r0"},
     {{cc, r7, r7, ASR, r7}, true, cc, "cc r7 r7 ASR r7", "cc_r7_r7_ASR_r7"},
     {{ls, r3, r3, ROR, r4}, true, ls, "ls r3 r3 ROR r4", "ls_r3_r3_ROR_r4"},
     {{hi, r0, r0, LSL, r0}, true, hi, "hi r0 r0 LSL r0", "hi_r0_r0_LSL_r0"},
     {{hi, r3, r3, LSL, r6}, true, hi, "hi r3 r3 LSL r6", "hi_r3_r3_LSL_r6"},
     {{le, r6, r6, ROR, r5}, true, le, "le r6 r6 ROR r5", "le_r6_r6_ROR_r5"},
     {{vs, r5, r5, ASR, r7}, true, vs, "vs r5 r5 ASR r7", "vs_r5_r5_ASR_r7"},
     {{pl, r5, r5, ROR, r5}, true, pl, "pl r5 r5 ROR r5", "pl_r5_r5_ROR_r5"},
     {{vc, r0, r0, LSR, r6}, true, vc, "vc r0 r0 LSR r6", "vc_r0_r0_LSR_r6"},
     {{ne, r2, r2, LSR, r3}, true, ne, "ne r2 r2 LSR r3", "ne_r2_r2_LSR_r3"},
     {{ls, r7, r7, ROR, r1}, true, ls, "ls r7 r7 ROR r1", "ls_r7_r7_ROR_r1"},
     {{ls, r2, r2, ROR, r3}, true, ls, "ls r2 r2 ROR r3", "ls_r2_r2_ROR_r3"},
     {{gt, r4, r4, LSR, r7}, true, gt, "gt r4 r4 LSR r7", "gt_r4_r4_LSR_r7"},
     {{hi, r4, r4, ASR, r4}, true, hi, "hi r4 r4 ASR r4", "hi_r4_r4_ASR_r4"},
     {{hi, r6, r6, ROR, r3}, true, hi, "hi r6 r6 ROR r3", "hi_r6_r6_ROR_r3"},
     {{vs, r5, r5, LSL, r3}, true, vs, "vs r5 r5 LSL r3", "vs_r5_r5_LSL_r3"},
     {{cs, r0, r0, LSR, r3}, true, cs, "cs r0 r0 LSR r3", "cs_r0_r0_LSR_r3"},
     {{cc, r4, r4, LSL, r7}, true, cc, "cc r4 r4 LSL r7", "cc_r4_r4_LSL_r7"},
     {{eq, r4, r4, ROR, r1}, true, eq, "eq r4 r4 ROR r1", "eq_r4_r4_ROR_r1"},
     {{vs, r7, r7, ASR, r3}, true, vs, "vs r7 r7 ASR r3", "vs_r7_r7_ASR_r3"},
     {{le, r5, r5, LSR, r7}, true, le, "le r5 r5 LSR r7", "le_r5_r5_LSR_r7"},
     {{lt, r7, r7, LSL, r1}, true, lt, "lt r7 r7 LSL r1", "lt_r7_r7_LSL_r1"},
     {{gt, r6, r6, ROR, r2}, true, gt, "gt r6 r6 ROR r2", "gt_r6_r6_ROR_r2"},
     {{vc, r0, r0, ASR, r0}, true, vc, "vc r0 r0 ASR r0", "vc_r0_r0_ASR_r0"},
     {{ls, r4, r4, LSR, r4}, true, ls, "ls r4 r4 LSR r4", "ls_r4_r4_LSR_r4"},
     {{hi, r4, r4, LSL, r5}, true, hi, "hi r4 r4 LSL r5", "hi_r4_r4_LSL_r5"},
     {{hi, r6, r6, LSR, r0}, true, hi, "hi r6 r6 LSR r0", "hi_r6_r6_LSR_r0"},
     {{ge, r3, r3, ASR, r0}, true, ge, "ge r3 r3 ASR r0", "ge_r3_r3_ASR_r0"},
     {{vs, r7, r7, ASR, r7}, true, vs, "vs r7 r7 ASR r7", "vs_r7_r7_ASR_r7"},
     {{hi, r1, r1, LSL, r3}, true, hi, "hi r1 r1 LSL r3", "hi_r1_r1_LSL_r3"},
     {{ls, r4, r4, ASR, r3}, true, ls, "ls r4 r4 ASR r3", "ls_r4_r4_ASR_r3"},
     {{eq, r7, r7, ASR, r6}, true, eq, "eq r7 r7 ASR r6", "eq_r7_r7_ASR_r6"},
     {{cc, r1, r1, ASR, r7}, true, cc, "cc r1 r1 ASR r7", "cc_r1_r1_ASR_r7"},
     {{hi, r4, r4, ROR, r1}, true, hi, "hi r4 r4 ROR r1", "hi_r4_r4_ROR_r1"},
     {{gt, r0, r0, ROR, r0}, true, gt, "gt r0 r0 ROR r0", "gt_r0_r0_ROR_r0"},
     {{hi, r0, r0, LSR, r3}, true, hi, "hi r0 r0 LSR r3", "hi_r0_r0_LSR_r3"},
     {{cc, r0, r0, LSL, r3}, true, cc, "cc r0 r0 LSL r3", "cc_r0_r0_LSL_r3"},
     {{ls, r6, r6, LSL, r6}, true, ls, "ls r6 r6 LSL r6", "ls_r6_r6_LSL_r6"},
     {{cs, r2, r2, LSR, r5}, true, cs, "cs r2 r2 LSR r5", "cs_r2_r2_LSR_r5"},
     {{le, r5, r5, LSL, r3}, true, le, "le r5 r5 LSL r3", "le_r5_r5_LSL_r3"},
     {{ge, r1, r1, LSR, r7}, true, ge, "ge r1 r1 LSR r7", "ge_r1_r1_LSR_r7"},
     {{cs, r4, r4, ASR, r7}, true, cs, "cs r4 r4 ASR r7", "cs_r4_r4_ASR_r7"},
     {{lt, r1, r1, ASR, r7}, true, lt, "lt r1 r1 ASR r7", "lt_r1_r1_ASR_r7"},
     {{mi, r4, r4, LSR, r5}, true, mi, "mi r4 r4 LSR r5", "mi_r4_r4_LSR_r5"},
     {{cc, r6, r6, ROR, r0}, true, cc, "cc r6 r6 ROR r0", "cc_r6_r6_ROR_r0"},
     {{mi, r4, r4, LSR, r1}, true, mi, "mi r4 r4 LSR r1", "mi_r4_r4_LSR_r1"},
     {{gt, r1, r1, ROR, r5}, true, gt, "gt r1 r1 ROR r5", "gt_r1_r1_ROR_r5"},
     {{vs, r2, r2, LSR, r2}, true, vs, "vs r2 r2 LSR r2", "vs_r2_r2_LSR_r2"},
     {{cs, r3, r3, LSL, r4}, true, cs, "cs r3 r3 LSL r4", "cs_r3_r3_LSL_r4"},
     {{le, r5, r5, ASR, r5}, true, le, "le r5 r5 ASR r5", "le_r5_r5_ASR_r5"},
     {{vs, r1, r1, LSR, r0}, true, vs, "vs r1 r1 LSR r0", "vs_r1_r1_LSR_r0"},
     {{le, r7, r7, LSL, r5}, true, le, "le r7 r7 LSL r5", "le_r7_r7_LSL_r5"},
     {{ne, r6, r6, LSR, r4}, true, ne, "ne r6 r6 LSR r4", "ne_r6_r6_LSR_r4"},
     {{le, r0, r0, ASR, r4}, true, le, "le r0 r0 ASR r4", "le_r0_r0_ASR_r4"},
     {{ls, r4, r4, ASR, r1}, true, ls, "ls r4 r4 ASR r1", "ls_r4_r4_ASR_r1"},
     {{gt, r1, r1, ASR, r6}, true, gt, "gt r1 r1 ASR r6", "gt_r1_r1_ASR_r6"},
     {{cs, r6, r6, LSL, r2}, true, cs, "cs r6 r6 LSL r2", "cs_r6_r6_LSL_r2"},
     {{cc, r2, r2, ROR, r0}, true, cc, "cc r2 r2 ROR r0", "cc_r2_r2_ROR_r0"},
     {{cc, r7, r7, ASR, r1}, true, cc, "cc r7 r7 ASR r1", "cc_r7_r7_ASR_r1"},
     {{ls, r6, r6, ASR, r0}, true, ls, "ls r6 r6 ASR r0", "ls_r6_r6_ASR_r0"},
     {{ls, r1, r1, LSL, r5}, true, ls, "ls r1 r1 LSL r5", "ls_r1_r1_LSL_r5"},
     {{le, r0, r0, LSR, r5}, true, le, "le r0 r0 LSR r5", "le_r0_r0_LSR_r5"},
     {{vs, r3, r3, LSR, r5}, true, vs, "vs r3 r3 LSR r5", "vs_r3_r3_LSR_r5"},
     {{vs, r3, r3, LSL, r1}, true, vs, "vs r3 r3 LSL r1", "vs_r3_r3_LSL_r1"},
     {{cc, r3, r3, ROR, r3}, true, cc, "cc r3 r3 ROR r3", "cc_r3_r3_ROR_r3"},
     {{le, r4, r4, ASR, r4}, true, le, "le r4 r4 ASR r4", "le_r4_r4_ASR_r4"},
     {{vs, r4, r4, ASR, r1}, true, vs, "vs r4 r4 ASR r1", "vs_r4_r4_ASR_r1"},
     {{pl, r2, r2, LSR, r0}, true, pl, "pl r2 r2 LSR r0", "pl_r2_r2_LSR_r0"},
     {{vs, r1, r1, LSL, r0}, true, vs, "vs r1 r1 LSL r0", "vs_r1_r1_LSL_r0"},
     {{cs, r6, r6, ASR, r7}, true, cs, "cs r6 r6 ASR r7", "cs_r6_r6_ASR_r7"},
     {{cs, r3, r3, ROR, r2}, true, cs, "cs r3 r3 ROR r2", "cs_r3_r3_ROR_r2"},
     {{mi, r4, r4, ROR, r7}, true, mi, "mi r4 r4 ROR r7", "mi_r4_r4_ROR_r7"},
     {{eq, r5, r5, ROR, r0}, true, eq, "eq r5 r5 ROR r0", "eq_r5_r5_ROR_r0"},
     {{cs, r3, r3, ROR, r0}, true, cs, "cs r3 r3 ROR r0", "cs_r3_r3_ROR_r0"},
     {{mi, r6, r6, LSR, r4}, true, mi, "mi r6 r6 LSR r4", "mi_r6_r6_LSR_r4"},
     {{hi, r2, r2, LSL, r4}, true, hi, "hi r2 r2 LSL r4", "hi_r2_r2_LSL_r4"},
     {{ge, r7, r7, LSR, r3}, true, ge, "ge r7 r7 LSR r3", "ge_r7_r7_LSR_r3"},
     {{mi, r0, r0, LSR, r6}, true, mi, "mi r0 r0 LSR r6", "mi_r0_r0_LSR_r6"},
     {{ne, r0, r0, ASR, r1}, true, ne, "ne r0 r0 ASR r1", "ne_r0_r0_ASR_r1"},
     {{ls, r0, r0, ROR, r3}, true, ls, "ls r0 r0 ROR r3", "ls_r0_r0_ROR_r3"},
     {{le, r1, r1, LSR, r4}, true, le, "le r1 r1 LSR r4", "le_r1_r1_LSR_r4"},
     {{ls, r3, r3, ROR, r5}, true, ls, "ls r3 r3 ROR r5", "ls_r3_r3_ROR_r5"},
     {{lt, r6, r6, ASR, r6}, true, lt, "lt r6 r6 ASR r6", "lt_r6_r6_ASR_r6"},
     {{cs, r5, r5, ASR, r3}, true, cs, "cs r5 r5 ASR r3", "cs_r5_r5_ASR_r3"},
     {{ls, r0, r0, LSR, r0}, true, ls, "ls r0 r0 LSR r0", "ls_r0_r0_LSR_r0"},
     {{ls, r5, r5, ASR, r4}, true, ls, "ls r5 r5 ASR r4", "ls_r5_r5_ASR_r4"},
     {{gt, r2, r2, ROR, r3}, true, gt, "gt r2 r2 ROR r3", "gt_r2_r2_ROR_r3"},
     {{hi, r4, r4, ASR, r3}, true, hi, "hi r4 r4 ASR r3", "hi_r4_r4_ASR_r3"},
     {{lt, r2, r2, ROR, r0}, true, lt, "lt r2 r2 ROR r0", "lt_r2_r2_ROR_r0"},
     {{ge, r0, r0, LSR, r3}, true, ge, "ge r0 r0 LSR r3", "ge_r0_r0_LSR_r3"},
     {{pl, r1, r1, ROR, r7}, true, pl, "pl r1 r1 ROR r7", "pl_r1_r1_ROR_r7"},
     {{gt, r0, r0, ROR, r5}, true, gt, "gt r0 r0 ROR r5", "gt_r0_r0_ROR_r5"},
     {{ls, r5, r5, LSR, r5}, true, ls, "ls r5 r5 LSR r5", "ls_r5_r5_LSR_r5"},
     {{cs, r3, r3, LSL, r5}, true, cs, "cs r3 r3 LSL r5", "cs_r3_r3_LSL_r5"},
     {{cs, r1, r1, LSL, r2}, true, cs, "cs r1 r1 LSL r2", "cs_r1_r1_LSL_r2"},
     {{ge, r5, r5, LSL, r3}, true, ge, "ge r5 r5 LSL r3", "ge_r5_r5_LSL_r3"},
     {{ls, r6, r6, ASR, r2}, true, ls, "ls r6 r6 ASR r2", "ls_r6_r6_ASR_r2"},
     {{pl, r7, r7, LSR, r7}, true, pl, "pl r7 r7 LSR r7", "pl_r7_r7_LSR_r7"},
     {{gt, r7, r7, LSR, r3}, true, gt, "gt r7 r7 LSR r3", "gt_r7_r7_LSR_r3"},
     {{ne, r5, r5, LSR, r5}, true, ne, "ne r5 r5 LSR r5", "ne_r5_r5_LSR_r5"},
     {{ne, r3, r3, ASR, r6}, true, ne, "ne r3 r3 ASR r6", "ne_r3_r3_ASR_r6"},
     {{eq, r3, r3, ASR, r6}, true, eq, "eq r3 r3 ASR r6", "eq_r3_r3_ASR_r6"},
     {{pl, r1, r1, ROR, r1}, true, pl, "pl r1 r1 ROR r1", "pl_r1_r1_ROR_r1"},
     {{lt, r7, r7, ROR, r6}, true, lt, "lt r7 r7 ROR r6", "lt_r7_r7_ROR_r6"},
     {{pl, r4, r4, ROR, r5}, true, pl, "pl r4 r4 ROR r5", "pl_r4_r4_ROR_r5"},
     {{ls, r7, r7, ASR, r0}, true, ls, "ls r7 r7 ASR r0", "ls_r7_r7_ASR_r0"},
     {{cs, r1, r1, ROR, r4}, true, cs, "cs r1 r1 ROR r4", "cs_r1_r1_ROR_r4"},
     {{eq, r0, r0, LSL, r1}, true, eq, "eq r0 r0 LSL r1", "eq_r0_r0_LSL_r1"},
     {{ls, r7, r7, ASR, r1}, true, ls, "ls r7 r7 ASR r1", "ls_r7_r7_ASR_r1"},
     {{le, r2, r2, LSL, r6}, true, le, "le r2 r2 LSL r6", "le_r2_r2_LSL_r6"},
     {{pl, r3, r3, ROR, r7}, true, pl, "pl r3 r3 ROR r7", "pl_r3_r3_ROR_r7"},
     {{lt, r7, r7, LSL, r0}, true, lt, "lt r7 r7 LSL r0", "lt_r7_r7_LSL_r0"},
     {{cc, r1, r1, LSL, r0}, true, cc, "cc r1 r1 LSL r0", "cc_r1_r1_LSL_r0"},
     {{mi, r0, r0, LSR, r3}, true, mi, "mi r0 r0 LSR r3", "mi_r0_r0_LSR_r3"},
     {{ls, r0, r0, LSL, r3}, true, ls, "ls r0 r0 LSL r3", "ls_r0_r0_LSL_r3"},
     {{lt, r4, r4, LSR, r6}, true, lt, "lt r4 r4 LSR r6", "lt_r4_r4_LSR_r6"},
     {{eq, r2, r2, ASR, r5}, true, eq, "eq r2 r2 ASR r5", "eq_r2_r2_ASR_r5"},
     {{ls, r6, r6, ROR, r0}, true, ls, "ls r6 r6 ROR r0", "ls_r6_r6_ROR_r0"},
     {{vc, r1, r1, ASR, r1}, true, vc, "vc r1 r1 ASR r1", "vc_r1_r1_ASR_r1"},
     {{pl, r0, r0, LSL, r6}, true, pl, "pl r0 r0 LSL r6", "pl_r0_r0_LSL_r6"},
     {{hi, r4, r4, LSL, r1}, true, hi, "hi r4 r4 LSL r1", "hi_r4_r4_LSL_r1"},
     {{vc, r7, r7, LSL, r2}, true, vc, "vc r7 r7 LSL r2", "vc_r7_r7_LSL_r2"},
     {{eq, r1, r1, LSR, r4}, true, eq, "eq r1 r1 LSR r4", "eq_r1_r1_LSR_r4"},
     {{hi, r2, r2, ASR, r0}, true, hi, "hi r2 r2 ASR r0", "hi_r2_r2_ASR_r0"},
     {{pl, r2, r2, LSL, r0}, true, pl, "pl r2 r2 LSL r0", "pl_r2_r2_LSL_r0"},
     {{lt, r2, r2, ASR, r3}, true, lt, "lt r2 r2 ASR r3", "lt_r2_r2_ASR_r3"},
     {{pl, r5, r5, ASR, r5}, true, pl, "pl r5 r5 ASR r5", "pl_r5_r5_ASR_r5"},
     {{le, r7, r7, LSL, r4}, true, le, "le r7 r7 LSL r4", "le_r7_r7_LSL_r4"},
     {{eq, r4, r4, ROR, r5}, true, eq, "eq r4 r4 ROR r5", "eq_r4_r4_ROR_r5"},
     {{vc, r4, r4, LSR, r2}, true, vc, "vc r4 r4 LSR r2", "vc_r4_r4_LSR_r2"},
     {{ne, r3, r3, ROR, r7}, true, ne, "ne r3 r3 ROR r7", "ne_r3_r3_ROR_r7"},
     {{gt, r3, r3, ROR, r4}, true, gt, "gt r3 r3 ROR r4", "gt_r3_r3_ROR_r4"},
     {{cs, r1, r1, ASR, r4}, true, cs, "cs r1 r1 ASR r4", "cs_r1_r1_ASR_r4"},
     {{cs, r4, r4, LSL, r3}, true, cs, "cs r4 r4 LSL r3", "cs_r4_r4_LSL_r3"},
     {{lt, r6, r6, ASR, r1}, true, lt, "lt r6 r6 ASR r1", "lt_r6_r6_ASR_r1"},
     {{hi, r3, r3, LSL, r5}, true, hi, "hi r3 r3 LSL r5", "hi_r3_r3_LSL_r5"},
     {{mi, r6, r6, ASR, r6}, true, mi, "mi r6 r6 ASR r6", "mi_r6_r6_ASR_r6"},
     {{le, r4, r4, LSR, r3}, true, le, "le r4 r4 LSR r3", "le_r4_r4_LSR_r3"},
     {{lt, r2, r2, LSL, r1}, true, lt, "lt r2 r2 LSL r1", "lt_r2_r2_LSL_r1"},
     {{ne, r5, r5, LSR, r2}, true, ne, "ne r5 r5 LSR r2", "ne_r5_r5_LSR_r2"},
     {{mi, r3, r3, ASR, r2}, true, mi, "mi r3 r3 ASR r2", "mi_r3_r3_ASR_r2"},
     {{ge, r0, r0, LSR, r2}, true, ge, "ge r0 r0 LSR r2", "ge_r0_r0_LSR_r2"},
     {{ls, r6, r6, LSR, r3}, true, ls, "ls r6 r6 LSR r3", "ls_r6_r6_LSR_r3"},
     {{cc, r2, r2, LSR, r0}, true, cc, "cc r2 r2 LSR r0", "cc_r2_r2_LSR_r0"},
     {{vs, r7, r7, LSL, r7}, true, vs, "vs r7 r7 LSL r7", "vs_r7_r7_LSL_r7"},
     {{vc, r7, r7, ASR, r7}, true, vc, "vc r7 r7 ASR r7", "vc_r7_r7_ASR_r7"},
     {{eq, r6, r6, LSR, r0}, true, eq, "eq r6 r6 LSR r0", "eq_r6_r6_LSR_r0"},
     {{vc, r6, r6, ASR, r3}, true, vc, "vc r6 r6 ASR r3", "vc_r6_r6_ASR_r3"},
     {{gt, r4, r4, ROR, r4}, true, gt, "gt r4 r4 ROR r4", "gt_r4_r4_ROR_r4"},
     {{ls, r5, r5, LSL, r4}, true, ls, "ls r5 r5 LSL r4", "ls_r5_r5_LSL_r4"},
     {{ge, r1, r1, ROR, r1}, true, ge, "ge r1 r1 ROR r1", "ge_r1_r1_ROR_r1"},
     {{cc, r5, r5, ASR, r0}, true, cc, "cc r5 r5 ASR r0", "cc_r5_r5_ASR_r0"},
     {{gt, r2, r2, LSL, r3}, true, gt, "gt r2 r2 LSL r3", "gt_r2_r2_LSL_r3"},
     {{ls, r0, r0, LSL, r5}, true, ls, "ls r0 r0 LSL r5", "ls_r0_r0_LSL_r5"},
     {{lt, r3, r3, LSL, r4}, true, lt, "lt r3 r3 LSL r4", "lt_r3_r3_LSL_r4"},
     {{eq, r6, r6, LSR, r3}, true, eq, "eq r6 r6 LSR r3", "eq_r6_r6_LSR_r3"},
     {{ne, r1, r1, ROR, r2}, true, ne, "ne r1 r1 ROR r2", "ne_r1_r1_ROR_r2"},
     {{vc, r2, r2, ROR, r0}, true, vc, "vc r2 r2 ROR r0", "vc_r2_r2_ROR_r0"},
     {{ls, r0, r0, LSR, r7}, true, ls, "ls r0 r0 LSR r7", "ls_r0_r0_LSR_r7"},
     {{mi, r3, r3, LSR, r2}, true, mi, "mi r3 r3 LSR r2", "mi_r3_r3_LSR_r2"},
     {{hi, r1, r1, LSL, r4}, true, hi, "hi r1 r1 LSL r4", "hi_r1_r1_LSL_r4"},
     {{cc, r1, r1, ROR, r3}, true, cc, "cc r1 r1 ROR r3", "cc_r1_r1_ROR_r3"},
     {{eq, r4, r4, ROR, r0}, true, eq, "eq r4 r4 ROR r0", "eq_r4_r4_ROR_r0"},
     {{vc, r6, r6, ASR, r2}, true, vc, "vc r6 r6 ASR r2", "vc_r6_r6_ASR_r2"},
     {{ls, r4, r4, LSR, r7}, true, ls, "ls r4 r4 LSR r7", "ls_r4_r4_LSR_r7"},
     {{vc, r1, r1, LSR, r1}, true, vc, "vc r1 r1 LSR r1", "vc_r1_r1_LSR_r1"},
     {{mi, r4, r4, ROR, r2}, true, mi, "mi r4 r4 ROR r2", "mi_r4_r4_ROR_r2"},
     {{ge, r7, r7, ROR, r5}, true, ge, "ge r7 r7 ROR r5", "ge_r7_r7_ROR_r5"},
     {{mi, r5, r5, LSR, r1}, true, mi, "mi r5 r5 LSR r1", "mi_r5_r5_LSR_r1"},
     {{cc, r3, r3, ASR, r6}, true, cc, "cc r3 r3 ASR r6", "cc_r3_r3_ASR_r6"},
     {{vc, r6, r6, ROR, r5}, true, vc, "vc r6 r6 ROR r5", "vc_r6_r6_ROR_r5"},
     {{mi, r7, r7, LSL, r0}, true, mi, "mi r7 r7 LSL r0", "mi_r7_r7_LSL_r0"},
     {{mi, r5, r5, LSR, r4}, true, mi, "mi r5 r5 LSR r4", "mi_r5_r5_LSR_r4"},
     {{lt, r6, r6, LSL, r5}, true, lt, "lt r6 r6 LSL r5", "lt_r6_r6_LSL_r5"},
     {{ne, r0, r0, ASR, r6}, true, ne, "ne r0 r0 ASR r6", "ne_r0_r0_ASR_r6"},
     {{le, r2, r2, ROR, r6}, true, le, "le r2 r2 ROR r6", "le_r2_r2_ROR_r6"},
     {{cc, r7, r7, LSL, r3}, true, cc, "cc r7 r7 LSL r3", "cc_r7_r7_LSL_r3"},
     {{cc, r6, r6, ASR, r6}, true, cc, "cc r6 r6 ASR r6", "cc_r6_r6_ASR_r6"},
     {{le, r2, r2, ASR, r3}, true, le, "le r2 r2 ASR r3", "le_r2_r2_ASR_r3"},
     {{ne, r6, r6, LSL, r2}, true, ne, "ne r6 r6 LSL r2", "ne_r6_r6_LSL_r2"},
     {{pl, r5, r5, LSL, r0}, true, pl, "pl r5 r5 LSL r0", "pl_r5_r5_LSL_r0"},
     {{vs, r1, r1, ASR, r3}, true, vs, "vs r1 r1 ASR r3", "vs_r1_r1_ASR_r3"},
     {{le, r7, r7, LSR, r1}, true, le, "le r7 r7 LSR r1", "le_r7_r7_LSR_r1"},
     {{ge, r3, r3, ROR, r4}, true, ge, "ge r3 r3 ROR r4", "ge_r3_r3_ROR_r4"},
     {{le, r1, r1, LSL, r4}, true, le, "le r1 r1 LSL r4", "le_r1_r1_LSL_r4"},
     {{ge, r2, r2, LSL, r6}, true, ge, "ge r2 r2 LSL r6", "ge_r2_r2_LSL_r6"},
     {{cs, r3, r3, ROR, r3}, true, cs, "cs r3 r3 ROR r3", "cs_r3_r3_ROR_r3"},
     {{vs, r0, r0, ROR, r1}, true, vs, "vs r0 r0 ROR r1", "vs_r0_r0_ROR_r1"},
     {{gt, r3, r3, LSR, r3}, true, gt, "gt r3 r3 LSR r3", "gt_r3_r3_LSR_r3"},
     {{pl, r2, r2, ASR, r4}, true, pl, "pl r2 r2 ASR r4", "pl_r2_r2_ASR_r4"},
     {{vs, r0, r0, ROR, r3}, true, vs, "vs r0 r0 ROR r3", "vs_r0_r0_ROR_r3"},
     {{mi, r7, r7, ASR, r0}, true, mi, "mi r7 r7 ASR r0", "mi_r7_r7_ASR_r0"},
     {{le, r6, r6, LSR, r2}, true, le, "le r6 r6 LSR r2", "le_r6_r6_LSR_r2"},
     {{ne, r4, r4, ROR, r6}, true, ne, "ne r4 r4 ROR r6", "ne_r4_r4_ROR_r6"},
     {{gt, r5, r5, ASR, r4}, true, gt, "gt r5 r5 ASR r4", "gt_r5_r5_ASR_r4"},
     {{ne, r1, r1, LSL, r2}, true, ne, "ne r1 r1 LSL r2", "ne_r1_r1_LSL_r2"},
     {{cs, r7, r7, ASR, r7}, true, cs, "cs r7 r7 ASR r7", "cs_r7_r7_ASR_r7"},
     {{ls, r2, r2, ROR, r2}, true, ls, "ls r2 r2 ROR r2", "ls_r2_r2_ROR_r2"},
     {{le, r3, r3, ASR, r5}, true, le, "le r3 r3 ASR r5", "le_r3_r3_ASR_r5"},
     {{lt, r5, r5, LSL, r4}, true, lt, "lt r5 r5 LSL r4", "lt_r5_r5_LSL_r4"},
     {{le, r0, r0, LSL, r2}, true, le, "le r0 r0 LSL r2", "le_r0_r0_LSL_r2"},
     {{pl, r3, r3, LSL, r5}, true, pl, "pl r3 r3 LSL r5", "pl_r3_r3_LSL_r5"},
     {{ge, r0, r0, ASR, r5}, true, ge, "ge r0 r0 ASR r5", "ge_r0_r0_ASR_r5"},
     {{ls, r3, r3, ASR, r6}, true, ls, "ls r3 r3 ASR r6", "ls_r3_r3_ASR_r6"},
     {{cs, r4, r4, LSL, r5}, true, cs, "cs r4 r4 LSL r5", "cs_r4_r4_LSL_r5"},
     {{cc, r4, r4, LSL, r1}, true, cc, "cc r4 r4 LSL r1", "cc_r4_r4_LSL_r1"},
     {{cs, r6, r6, LSL, r7}, true, cs, "cs r6 r6 LSL r7", "cs_r6_r6_LSL_r7"},
     {{ne, r4, r4, LSL, r6}, true, ne, "ne r4 r4 LSL r6", "ne_r4_r4_LSL_r6"},
     {{cs, r7, r7, ASR, r1}, true, cs, "cs r7 r7 ASR r1", "cs_r7_r7_ASR_r1"},
     {{cc, r2, r2, ASR, r1}, true, cc, "cc r2 r2 ASR r1", "cc_r2_r2_ASR_r1"},
     {{ne, r6, r6, ASR, r2}, true, ne, "ne r6 r6 ASR r2", "ne_r6_r6_ASR_r2"},
     {{mi, r6, r6, LSL, r3}, true, mi, "mi r6 r6 LSL r3", "mi_r6_r6_LSL_r3"},
     {{lt, r6, r6, ASR, r0}, true, lt, "lt r6 r6 ASR r0", "lt_r6_r6_ASR_r0"},
     {{mi, r1, r1, LSL, r0}, true, mi, "mi r1 r1 LSL r0", "mi_r1_r1_LSL_r0"},
     {{pl, r1, r1, LSL, r0}, true, pl, "pl r1 r1 LSL r0", "pl_r1_r1_LSL_r0"},
     {{gt, r0, r0, LSL, r5}, true, gt, "gt r0 r0 LSL r5", "gt_r0_r0_LSL_r5"},
     {{gt, r4, r4, ASR, r3}, true, gt, "gt r4 r4 ASR r3", "gt_r4_r4_ASR_r3"},
     {{pl, r1, r1, LSR, r2}, true, pl, "pl r1 r1 LSR r2", "pl_r1_r1_LSR_r2"},
     {{vc, r1, r1, ROR, r6}, true, vc, "vc r1 r1 ROR r6", "vc_r1_r1_ROR_r6"},
     {{ge, r4, r4, ROR, r0}, true, ge, "ge r4 r4 ROR r0", "ge_r4_r4_ROR_r0"},
     {{le, r1, r1, LSR, r0}, true, le, "le r1 r1 LSR r0", "le_r1_r1_LSR_r0"},
     {{cs, r5, r5, LSL, r2}, true, cs, "cs r5 r5 LSL r2", "cs_r5_r5_LSL_r2"},
     {{ge, r5, r5, ROR, r1}, true, ge, "ge r5 r5 ROR r1", "ge_r5_r5_ROR_r1"},
     {{mi, r3, r3, ROR, r7}, true, mi, "mi r3 r3 ROR r7", "mi_r3_r3_ROR_r7"},
     {{ls, r3, r3, ASR, r2}, true, ls, "ls r3 r3 ASR r2", "ls_r3_r3_ASR_r2"},
     {{le, r1, r1, LSL, r3}, true, le, "le r1 r1 LSL r3", "le_r1_r1_LSL_r3"},
     {{ls, r2, r2, ASR, r4}, true, ls, "ls r2 r2 ASR r4", "ls_r2_r2_ASR_r4"},
     {{lt, r5, r5, ASR, r2}, true, lt, "lt r5 r5 ASR r2", "lt_r5_r5_ASR_r2"},
     {{cs, r0, r0, ASR, r0}, true, cs, "cs r0 r0 ASR r0", "cs_r0_r0_ASR_r0"},
     {{cs, r2, r2, ROR, r1}, true, cs, "cs r2 r2 ROR r1", "cs_r2_r2_ROR_r1"},
     {{hi, r2, r2, ROR, r1}, true, hi, "hi r2 r2 ROR r1", "hi_r2_r2_ROR_r1"},
     {{ge, r0, r0, ASR, r4}, true, ge, "ge r0 r0 ASR r4", "ge_r0_r0_ASR_r4"},
     {{le, r7, r7, LSR, r5}, true, le, "le r7 r7 LSR r5", "le_r7_r7_LSR_r5"},
     {{eq, r2, r2, ROR, r3}, true, eq, "eq r2 r2 ROR r3", "eq_r2_r2_ROR_r3"},
     {{hi, r2, r2, LSL, r1}, true, hi, "hi r2 r2 LSL r1", "hi_r2_r2_LSL_r1"},
     {{hi, r4, r4, LSR, r0}, true, hi, "hi r4 r4 LSR r0", "hi_r4_r4_LSR_r0"},
     {{gt, r6, r6, ASR, r5}, true, gt, "gt r6 r6 ASR r5", "gt_r6_r6_ASR_r5"},
     {{pl, r0, r0, ASR, r7}, true, pl, "pl r0 r0 ASR r7", "pl_r0_r0_ASR_r7"},
     {{cc, r3, r3, LSL, r2}, true, cc, "cc r3 r3 LSL r2", "cc_r3_r3_LSL_r2"},
     {{mi, r0, r0, ASR, r7}, true, mi, "mi r0 r0 ASR r7", "mi_r0_r0_ASR_r7"},
     {{eq, r4, r4, LSR, r7}, true, eq, "eq r4 r4 LSR r7", "eq_r4_r4_LSR_r7"},
     {{vs, r0, r0, ASR, r1}, true, vs, "vs r0 r0 ASR r1", "vs_r0_r0_ASR_r1"},
     {{ge, r1, r1, LSR, r2}, true, ge, "ge r1 r1 LSR r2", "ge_r1_r1_LSR_r2"},
     {{ge, r4, r4, LSR, r4}, true, ge, "ge r4 r4 LSR r4", "ge_r4_r4_LSR_r4"},
     {{le, r3, r3, LSL, r6}, true, le, "le r3 r3 LSL r6", "le_r3_r3_LSL_r6"},
     {{pl, r6, r6, ASR, r4}, true, pl, "pl r6 r6 ASR r4", "pl_r6_r6_ASR_r4"},
     {{ne, r3, r3, LSR, r6}, true, ne, "ne r3 r3 LSR r6", "ne_r3_r3_LSR_r6"},
     {{gt, r3, r3, LSR, r6}, true, gt, "gt r3 r3 LSR r6", "gt_r3_r3_LSR_r6"},
     {{lt, r1, r1, LSL, r3}, true, lt, "lt r1 r1 LSL r3", "lt_r1_r1_LSL_r3"},
     {{ne, r1, r1, LSR, r7}, true, ne, "ne r1 r1 LSR r7", "ne_r1_r1_LSR_r7"},
     {{ne, r1, r1, ROR, r4}, true, ne, "ne r1 r1 ROR r4", "ne_r1_r1_ROR_r4"},
     {{le, r7, r7, LSR, r7}, true, le, "le r7 r7 LSR r7", "le_r7_r7_LSR_r7"},
     {{lt, r1, r1, ASR, r5}, true, lt, "lt r1 r1 ASR r5", "lt_r1_r1_ASR_r5"},
     {{ls, r6, r6, LSR, r6}, true, ls, "ls r6 r6 LSR r6", "ls_r6_r6_LSR_r6"},
     {{gt, r2, r2, LSL, r5}, true, gt, "gt r2 r2 LSL r5", "gt_r2_r2_LSL_r5"},
     {{gt, r1, r1, LSL, r4}, true, gt, "gt r1 r1 LSL r4", "gt_r1_r1_LSL_r4"},
     {{gt, r0, r0, LSL, r0}, true, gt, "gt r0 r0 LSL r0", "gt_r0_r0_LSL_r0"},
     {{lt, r0, r0, ASR, r7}, true, lt, "lt r0 r0 ASR r7", "lt_r0_r0_ASR_r7"},
     {{ge, r0, r0, LSL, r5}, true, ge, "ge r0 r0 LSL r5", "ge_r0_r0_LSL_r5"},
     {{eq, r2, r2, LSR, r7}, true, eq, "eq r2 r2 LSR r7", "eq_r2_r2_LSR_r7"},
     {{pl, r2, r2, LSL, r4}, true, pl, "pl r2 r2 LSL r4", "pl_r2_r2_LSL_r4"},
     {{gt, r7, r7, LSR, r7}, true, gt, "gt r7 r7 LSR r7", "gt_r7_r7_LSR_r7"},
     {{lt, r5, r5, ROR, r6}, true, lt, "lt r5 r5 ROR r6", "lt_r5_r5_ROR_r6"},
     {{vc, r0, r0, LSL, r5}, true, vc, "vc r0 r0 LSL r5", "vc_r0_r0_LSL_r5"},
     {{cc, r2, r2, LSR, r3}, true, cc, "cc r2 r2 LSR r3", "cc_r2_r2_LSR_r3"},
     {{ne, r6, r6, LSR, r2}, true, ne, "ne r6 r6 LSR r2", "ne_r6_r6_LSR_r2"},
     {{mi, r6, r6, LSR, r7}, true, mi, "mi r6 r6 LSR r7", "mi_r6_r6_LSR_r7"},
     {{gt, r2, r2, LSR, r4}, true, gt, "gt r2 r2 LSR r4", "gt_r2_r2_LSR_r4"},
     {{pl, r6, r6, ASR, r5}, true, pl, "pl r6 r6 ASR r5", "pl_r6_r6_ASR_r5"},
     {{ge, r6, r6, ASR, r5}, true, ge, "ge r6 r6 ASR r5", "ge_r6_r6_ASR_r5"},
     {{pl, r0, r0, LSR, r1}, true, pl, "pl r0 r0 LSR r1", "pl_r0_r0_LSR_r1"},
     {{ls, r3, r3, ROR, r3}, true, ls, "ls r3 r3 ROR r3", "ls_r3_r3_ROR_r3"},
     {{gt, r7, r7, LSL, r3}, true, gt, "gt r7 r7 LSL r3", "gt_r7_r7_LSL_r3"},
     {{mi, r2, r2, LSR, r6}, true, mi, "mi r2 r2 LSR r6", "mi_r2_r2_LSR_r6"},
     {{ge, r0, r0, ROR, r4}, true, ge, "ge r0 r0 ROR r4", "ge_r0_r0_ROR_r4"},
     {{le, r5, r5, ROR, r6}, true, le, "le r5 r5 ROR r6", "le_r5_r5_ROR_r6"},
     {{mi, r1, r1, LSL, r1}, true, mi, "mi r1 r1 LSL r1", "mi_r1_r1_LSL_r1"},
     {{cc, r4, r4, LSL, r5}, true, cc, "cc r4 r4 LSL r5", "cc_r4_r4_LSL_r5"}};

// These headers each contain an array of `TestResult` with the reference output
// values. The reference arrays are names `kReference{mnemonic}`.
#include "aarch32/traces/assembler-cond-rd-operand-rn-shift-rs-in-it-block-mov-t32.h"


// The maximum number of errors to report in detail for each test.
const unsigned kErrorReportLimit = 8;

typedef void (MacroAssembler::*Fn)(Condition cond,
                                   Register rd,
                                   const Operand& op);

void TestHelper(Fn instruction,
                const char* mnemonic,
                const TestResult reference[]) {
  unsigned total_error_count = 0;
  MacroAssembler masm(BUF_SIZE);

  masm.UseT32();

  for (unsigned i = 0; i < ARRAY_SIZE(kTests); i++) {
    // Values to pass to the macro-assembler.
    Condition cond = kTests[i].operands.cond;
    Register rd = kTests[i].operands.rd;
    Register rn = kTests[i].operands.rn;
    ShiftType shift = kTests[i].operands.shift;
    Register rs = kTests[i].operands.rs;
    Operand op(rn, shift, rs);

    int32_t start = masm.GetCursorOffset();
    {
      // We never generate more that 4 bytes, as IT instructions are only
      // allowed for narrow encodings.
      ExactAssemblyScope scope(&masm, 4, ExactAssemblyScope::kMaximumSize);
      if (kTests[i].in_it_block) {
        masm.it(kTests[i].it_condition);
      }
      (masm.*instruction)(cond, rd, op);
    }
    int32_t end = masm.GetCursorOffset();

    const byte* result_ptr =
        masm.GetBuffer()->GetOffsetAddress<const byte*>(start);
    VIXL_ASSERT(start < end);
    uint32_t result_size = end - start;

    if (Test::generate_test_trace()) {
      // Print the result bytes.
      printf("const byte kInstruction_%s_%s[] = {\n",
             mnemonic,
             kTests[i].identifier);
      for (uint32_t j = 0; j < result_size; j++) {
        if (j == 0) {
          printf("  0x%02" PRIx8, result_ptr[j]);
        } else {
          printf(", 0x%02" PRIx8, result_ptr[j]);
        }
      }
      // This comment is meant to be used by external tools to validate
      // the encoding. We can parse the comment to figure out what
      // instruction this corresponds to.
      if (kTests[i].in_it_block) {
        printf(" // It %s; %s %s\n};\n",
               kTests[i].it_condition.GetName(),
               mnemonic,
               kTests[i].operands_description);
      } else {
        printf(" // %s %s\n};\n", mnemonic, kTests[i].operands_description);
      }
    } else {
      // Check we've emitted the exact same encoding as present in the
      // trace file. Only print up to `kErrorReportLimit` errors.
      if (((result_size != reference[i].size) ||
           (memcmp(result_ptr, reference[i].encoding, reference[i].size) !=
            0)) &&
          (++total_error_count <= kErrorReportLimit)) {
        printf("Error when testing \"%s\" with operands \"%s\":\n",
               mnemonic,
               kTests[i].operands_description);
        printf("  Expected: ");
        for (uint32_t j = 0; j < reference[i].size; j++) {
          if (j == 0) {
            printf("0x%02" PRIx8, reference[i].encoding[j]);
          } else {
            printf(", 0x%02" PRIx8, reference[i].encoding[j]);
          }
        }
        printf("\n");
        printf("  Found:    ");
        for (uint32_t j = 0; j < result_size; j++) {
          if (j == 0) {
            printf("0x%02" PRIx8, result_ptr[j]);
          } else {
            printf(", 0x%02" PRIx8, result_ptr[j]);
          }
        }
        printf("\n");
      }
    }
  }

  masm.FinalizeCode();

  if (Test::generate_test_trace()) {
    // Finalize the trace file by writing the final `TestResult` array
    // which links all generated instruction encodings.
    printf("const TestResult kReference%s[] = {\n", mnemonic);
    for (unsigned i = 0; i < ARRAY_SIZE(kTests); i++) {
      printf("  {\n");
      printf("    ARRAY_SIZE(kInstruction_%s_%s),\n",
             mnemonic,
             kTests[i].identifier);
      printf("    kInstruction_%s_%s,\n", mnemonic, kTests[i].identifier);
      printf("  },\n");
    }
    printf("};\n");
  } else {
    if (total_error_count > kErrorReportLimit) {
      printf("%u other errors follow.\n",
             total_error_count - kErrorReportLimit);
    }
    // Crash if the test failed.
    VIXL_CHECK(total_error_count == 0);
  }
}

// Instantiate tests for each instruction in the list.
#define TEST(mnemonic)                                                       \
  void Test_##mnemonic() {                                                   \
    TestHelper(&MacroAssembler::mnemonic, #mnemonic, kReference##mnemonic);  \
  }                                                                          \
  Test test_##mnemonic(                                                      \
      "AARCH32_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_RS_IN_IT_BLOCK_" #mnemonic \
      "_T32",                                                                \
      &Test_##mnemonic);
FOREACH_INSTRUCTION(TEST)
#undef TEST

}  // namespace
#endif

}  // namespace aarch32
}  // namespace vixl
